import json
from nose import SkipTest
from tests.test_stack import TestConfig, app_from_config
from tg.support.paginate import Page
from tg.controllers.util import _urlencode
from tg import json_encode
from tg.util.webtest import test_context


def setup_noDB():
    base_config = TestConfig(folder='rendering',
            values={
                'use_sqlalchemy': False,
                'use_toscawidgets': False,
                'use_toscawidgets2': False
            })
    return app_from_config(base_config)


_pager = ('<div id="pager"><span class="pager_curpage">1</span>'
    ' <a href="%(url)s?page=2">2</a>'
    ' <a href="%(url)s?page=3">3</a>'
    ' <span class="pager_dotdot">..</span>'
    ' <a href="%(url)s?page=5">5</a></div>')

_data = '<ul id="data">%s</ul>' % ''.join(
        '<li>%d</li>' % i for i in range(10))


class TestPagination:
    def setup(self):
        self.app = setup_noDB()

    def test_basic_pagination(self):
        url = '/paginated/42'
        page = self.app.get(url)
        assert _pager % locals() in page, page
        assert _data in page, page
        url = '/paginated/42?page=2'
        page = self.app.get(url)
        assert '<li>0</li>' not in page
        assert '<li>10</li>' in page

    def test_pagination_negative(self):
        url = '/paginated/42?page=-1'
        page = self.app.get(url)
        assert '<li>0</li>' in page

    def test_pagination_items_per_page(self):
        url = '/paginated/42?items_per_page=20'
        page = self.app.get(url)
        assert '<li>0</li>' in page
        assert '<li>19</li>' in page

    def test_pagination_items_per_page_negative(self):
        url = '/paginated/42?items_per_page=-1'
        page = self.app.get(url)
        assert '<li>0</li>' in page
        assert '<li>10</li>' not in page

    def test_pagination_non_paginable(self):
        url = '/paginated_text'
        page = self.app.get(url)
        assert 'Some Text' in page

    def test_pagination_with_validation(self):
        url = '/paginated_validated/42'
        page = self.app.get(url)
        assert _pager % locals() in page, page
        assert _data in page, page
        url = '/paginated_validated/42?page=2'
        page = self.app.get(url)
        assert '<li>0</li>' not in page
        assert '<li>10</li>' in page

    def test_validation_with_pagination(self):
        url = '/validated_paginated/42'
        page = self.app.get(url)
        assert _pager % locals() in page, page
        assert _data in page, page
        url = '/validated_paginated/42?page=2'
        page = self.app.get(url)
        assert '<li>0</li>' not in page
        assert '<li>10</li>' in page

    def test_pagination_with_link_args(self):
        url = '/paginate_with_params/42'
        page = self.app.get(url)
        assert 'param1=hi' in page
        assert 'param2=man' in page
        assert 'partial' not in page
        assert '/fake_url' in page
        url = '/paginate_with_params/42?page=2'
        page = self.app.get(url)
        assert '<li>0</li>' not in page
        assert '<li>10</li>' in page

    def test_multiple_paginators(self):
        url = '/multiple_paginators/42'

        try:
            from collections import OrderedDict
            params = (('testdata_page', 2), ('testdata2_page', 2))
            reverse_params = OrderedDict(reversed(params))
            params = OrderedDict(params)
        except ImportError:
            reverse_params = params = {'testdata2_page': 2, 'testdata_page': 2}

        goto_page2_link = url + '?' + _urlencode(params)
        goto_page2_reverse_link = url + '?' + _urlencode(reverse_params)

        page = self.app.get(url)
        assert '/multiple_paginators/42?testdata2_page=2' in page, str(page)
        assert '/multiple_paginators/42?testdata_page=2' in page, str(page)

        url = '/multiple_paginators/42?testdata_page=2'
        page = self.app.get(url)

        assert (
            goto_page2_link in page or goto_page2_reverse_link in page
        ), str(page)
        assert '/multiple_paginators/42?testdata_page=4' in page, str(page)

        assert '<li>0</li>' not in page
        assert '<li>10</li>' in page
        assert '<li>142</li>' in page
        assert '<li>151</li>' in page

        url = '/multiple_paginators/42?testdata2_page=2'
        page = self.app.get(url)

        assert (
            goto_page2_link in page or goto_page2_reverse_link in page
        ), str(page)
        assert '/multiple_paginators/42?testdata2_page=4' in page, str(page)

        assert '<li>0</li>' in page
        assert '<li>9</li>' in page
        assert '<li>151</li>' not in page
        assert '<li>161</li>' in page

    def test_json_pagination(self):
        url = '/paginated/42.json'
        page = self.app.get(url)
        assert '[0, 1, 2, 3, 4, 5, 6, 7, 8, 9]' in page

        url = '/paginated/42.json?page=2'
        page = self.app.get(url)
        assert '[10, 11, 12, 13, 14, 15, 16, 17, 18, 19]' in page


class TestPage(object):
    def test_not_a_number_page(self):
        p = Page(range(100), items_per_page=10, page='A')
        sec = list(p)
        assert sec[-1] == 9, sec

    def test_empty_list(self):
        p = Page([], items_per_page=10, page=1)
        assert list(p) == []

    def test_page_out_of_bound(self):
        p = Page(range(100), items_per_page=10, page=10000)
        sec = list(p)
        assert sec[-1] == 99, sec

    def test_page_out_of_lower_bound(self):
        p = Page(range(100), items_per_page=10, page=-5)
        sec = list(p)
        assert sec[-1] == 9, sec

    def test_navigator_one_page(self):
        with test_context(None, '/'):
            p = Page(range(10), items_per_page=10, page=10)
            assert p.pager() == ''

    def test_navigator_middle_page(self):
        with test_context(None, '/'):
            p = Page(range(100), items_per_page=10, page=5)
            pager = p.pager()

            assert '?page=1' in pager
            assert '?page=4' in pager
            assert '?page=6' in pager
            assert '?page=10' in pager

    def test_navigator_ajax(self):
        with test_context(None, '/'):
            p = Page(range(100), items_per_page=10, page=5)
            pager = p.pager(onclick='goto($page)')

            assert 'goto(1)' in pager
            assert 'goto(4)' in pager
            assert 'goto(6)' in pager
            assert 'goto(10)' in pager


try:
    import sqlite3
except:
    import pysqlite2
from sqlalchemy import (MetaData, Table, Column, ForeignKey, Integer, String)
from sqlalchemy.orm import create_session, mapper, relation

metadata = MetaData('sqlite:///:memory:')

test1 = Table('test1', metadata,
    Column('id', Integer, primary_key=True),
    Column('val', String(8)))

test2 = Table('test2', metadata,
    Column('id', Integer, primary_key=True),
    Column('test1id', Integer, ForeignKey('test1.id')),
    Column('val', String(8)))

test3 = Table('test3', metadata,
    Column('id', Integer, primary_key=True),
    Column('val', String(8)))

test4 = Table('test4', metadata,
    Column('id', Integer, primary_key=True),
    Column('val', String(8)))

metadata.create_all()

class Test2(object):
    pass
mapper(Test2, test2)

class Test1(object):
    pass
mapper(Test1, test1, properties={'test2s': relation(Test2)})

class Test3(object):
    pass
mapper(Test3, test3)

class Test4(object):
    pass
mapper(Test4, test4)

test1.insert().execute({'id': 1, 'val': 'bob'})
test2.insert().execute({'id': 1, 'test1id': 1, 'val': 'fred'})
test2.insert().execute({'id': 2, 'test1id': 1, 'val': 'alice'})
test3.insert().execute({'id': 1, 'val': 'bob'})
test4.insert().execute({'id': 1, 'val': 'alberto'})

class TestPageSQLA(object):
    def setup(self):
        self.s = create_session()

    def test_relationship(self):
        t = self.s.query(Test1).get(1)
        p = Page(t.test2s, items_per_page=1, page=1)
        assert len(list(p)) == 1
        assert list(p)[0].val == 'fred', list(p)

    def test_query(self):
        q = self.s.query(Test2)
        p = Page(q, items_per_page=1, page=1)
        assert len(list(p)) == 1
        assert list(p)[0].val == 'fred', list(p)

    def test_json_query(self):
        q = self.s.query(Test2)
        p = Page(q, items_per_page=1, page=1)
        res = json.loads(json_encode(p))
        assert len(res['entries']) == 1
        assert res['total'] == 2
        assert res['entries'][0]['val'] == 'fred'


try:
    import ming
    from ming import create_datastore, Session, schema, ASCENDING
    from ming.odm import ODMSession, FieldProperty, ForeignIdProperty, RelationProperty, Mapper
    from ming.odm.declarative import MappedClass
except ImportError:
    ming = None


class TestPageMing(object):
    @classmethod
    def setupClass(cls):
        if ming is None:
            raise SkipTest('Ming not available...')

        cls.basic_session = Session(create_datastore('mim:///testdb'))
        cls.s = ODMSession(cls.basic_session)

        class Author(MappedClass):
            class __mongometa__:
                session = cls.s
                name = 'wiki_author'

            _id = FieldProperty(schema.ObjectId)
            name = FieldProperty(str)
            pages = RelationProperty('WikiPage')

        class WikiPage(MappedClass):
            class __mongometa__:
                session = cls.s
                name = 'wiki_page'

            _id = FieldProperty(schema.ObjectId)
            title = FieldProperty(str)
            text = FieldProperty(str)
            order = FieldProperty(int)
            author_id = ForeignIdProperty(Author)
            author = RelationProperty(Author)

        cls.Author = Author
        cls.WikiPage = WikiPage
        Mapper.compile_all()

        cls.author = Author(name='author1')
        author2 = Author(name='author2')

        WikiPage(title='Hello', text='Text', order=1, author=cls.author)
        WikiPage(title='Another', text='Text', order=2, author=cls.author)
        WikiPage(title='ThirdOne', text='Text', order=3, author=author2)
        cls.s.flush()
        cls.s.clear()

    def teardown(self):
        self.s.clear()

    def test_query(self):
        q = self.WikiPage.query.find().sort([('order', ASCENDING)])
        p = Page(q, items_per_page=1, page=1)
        assert len(list(p)) == 1
        assert list(p)[0].title == 'Hello', list(p)

    def test_json_query(self):
        q = self.WikiPage.query.find().sort([('order', ASCENDING)])
        p = Page(q, items_per_page=1, page=1)
        res = json.loads(json_encode(p))
        assert len(res['entries']) == 1
        assert res['total'] == 3
        assert res['entries'][0]['title'] == 'Hello', res['entries']
        assert res['entries'][0]['author_id'] == str(self.author._id), res['entries']

    def test_relation(self):
        a = self.Author.query.find({'name': 'author1'}).first()
        p = Page(a.pages, items_per_page=1, page=1)
        assert len(list(p)) == 1
        assert list(p)[0].title in ('Hello', 'Another'), list(p)



# -*- coding: utf-8 -*-
from nose.tools import raises

from tg.configuration.utils import DependenciesList

class DLEntry1: pass
class DLEntry2: pass
class DLEntry3: pass
class DLEntry4: pass
class DLEntry5: pass


class TestDependenciesList(object):
    def test_basic_with_classes(self):
        dl = DependenciesList()

        dl.add(DLEntry2)
        dl.add(DLEntry4, after=DLEntry3)
        dl.add(DLEntry3)
        dl.add(DLEntry1, after=False)
        dl.add(DLEntry5, after=DLEntry3)

        dl_values = list(dl.values())
        assert dl_values[0] == DLEntry1
        assert dl_values[1] == DLEntry2
        assert dl_values[2] == DLEntry3
        assert dl_values[3] == DLEntry4
        assert dl_values[4] == DLEntry5

    def test_basic_iter(self):
        dl = DependenciesList()

        dl.add(DLEntry2)
        dl.add(DLEntry4, after=DLEntry3)
        dl.add(DLEntry3)
        dl.add(DLEntry1, after=False)
        dl.add(DLEntry5, after=DLEntry3)

        visisted = []
        for key, value in dl:
            visisted.append(key)

        assert visisted == ['DLEntry1', 'DLEntry2', 'DLEntry3', 'DLEntry4', 'DLEntry5']

    def test_basic_repr(self):
        dl = DependenciesList()

        dl.add(DLEntry2)
        dl.add(DLEntry4, after=DLEntry3)
        dl.add(DLEntry3)
        dl.add(DLEntry1, after=False)
        dl.add(DLEntry5, after=DLEntry3)

        expected = "<DependenciesList ['DLEntry1', 'DLEntry2', 'DLEntry3', 'DLEntry4', 'DLEntry5']>"
        assert repr(dl) == expected

    def test_basic_with_ids(self):
        dl = DependenciesList()

        dl.add(DLEntry2, 'num2')
        dl.add(DLEntry4, 'num4', after='num3')
        dl.add(DLEntry3, 'num3')
        dl.add(DLEntry1, 'num1', after=False)
        dl.add(DLEntry5, 'num5', after='num3')

        dl_values = list(dl.values())
        assert dl_values[0] == DLEntry1
        assert dl_values[1] == DLEntry2
        assert dl_values[2] == DLEntry3
        assert dl_values[3] == DLEntry4
        assert dl_values[4] == DLEntry5

    def test_reversed_with_ids(self):
        dl = DependenciesList()

        dl.add(DLEntry5, 'num5', after='num4')
        dl.add(DLEntry4, 'num4', after='num3')
        dl.add(DLEntry3, 'num3', after='num2')
        dl.add(DLEntry2, 'num2', after='num1')
        dl.add(DLEntry1, 'num1')

        dl_values = list(dl.values())
        assert dl_values[0] == DLEntry1
        assert dl_values[1] == DLEntry2
        assert dl_values[2] == DLEntry3
        assert dl_values[3] == DLEntry4
        assert dl_values[4] == DLEntry5

    def test_multiple_with_ids(self):
        dl = DependenciesList()
        dl.add(DLEntry1)
        dl.add(DLEntry4, after=DLEntry3)
        dl.add(DLEntry5, after=DLEntry4)
        dl.add(DLEntry2, after=DLEntry1)
        dl.add(DLEntry3, after=DLEntry1)

        dl_values = list(dl.values())
        assert dl_values[0] == DLEntry1
        assert dl_values[1] == DLEntry2
        assert dl_values[2] == DLEntry3
        assert dl_values[3] == DLEntry4
        assert dl_values[4] == DLEntry5

    def test_multiple_with_missing_step(self):
        dl = DependenciesList()
        dl.add(DLEntry1)
        dl.add(DLEntry3, after=DLEntry2)
        dl.add(DLEntry2, after=DLEntry5)

        dl_values = list(dl.values())
        assert dl_values[0] == DLEntry1
        assert dl_values[1] == DLEntry2
        assert dl_values[2] == DLEntry3

    def test_objects_instead_of_classes(self):
        dl = DependenciesList()
        dl.add(DLEntry1(), 'DLEntry1')
        dl.add(DLEntry3(), 'DLEntry3', after=DLEntry2)
        dl.add(DLEntry2(), 'DLEntry2', after=DLEntry5)

        dl_values = list(dl.values())
        assert dl_values[0].__class__ == DLEntry1
        assert dl_values[1].__class__ == DLEntry2
        assert dl_values[2].__class__ == DLEntry3

    @raises(ValueError)
    def test_objects_must_have_key(self):
        dl = DependenciesList()
        dl.add(DLEntry1())

    @raises(ValueError)
    def test_after_cannot_be_an_instance(self):
        dl = DependenciesList()
        dl.add(DLEntry1(), key='DLEntry1')
        dl.add(DLEntry2(), key='DLEntry2', after=DLEntry1())

    def test_after_everything_else(self):
        dl = DependenciesList()

        dl.add(DLEntry2, after=True)
        dl.add(DLEntry5, after=True)
        dl.add(DLEntry4, after=DLEntry3)
        dl.add(DLEntry3, after=DLEntry2)

        dl.add(DLEntry1)

        dl_values = list(dl.values())
        assert dl_values[0] == DLEntry1
        assert dl_values[1] == DLEntry2
        assert dl_values[2] == DLEntry3
        assert dl_values[3] == DLEntry4
        assert dl_values[4] == DLEntry5

    def test_replacing_elements_with_key(self):
        dl = DependenciesList()

        dl.add(DLEntry2, 'num2')
        dl.add(DLEntry4, 'num4', after='num3')
        dl.add(DLEntry3, 'num3')
        dl.add(DLEntry1, 'num1', after=False)
        dl.add(DLEntry5, 'num5', after='num3')

        dl.replace('num3', DLEntry1)

        dl_values = list(dl.values())
        assert dl_values[0] == DLEntry1
        assert dl_values[1] == DLEntry2
        assert dl_values[2] == DLEntry1
        assert dl_values[3] == DLEntry4
        assert dl_values[4] == DLEntry5

    def test_replacing_elements_with_classes(self):
        dl = DependenciesList()

        dl.add(DLEntry2)
        dl.add(DLEntry4, after=DLEntry3)
        dl.add(DLEntry3)
        dl.add(DLEntry1, after=False)
        dl.add(DLEntry5, after=DLEntry3)

        dl.replace(DLEntry3, DLEntry1)

        dl_values = list(dl.values())
        assert dl_values[0] == DLEntry1
        assert dl_values[1] == DLEntry2
        assert dl_values[2] == DLEntry1
        assert dl_values[3] == DLEntry4
        assert dl_values[4] == DLEntry5

    @raises(ValueError)
    def test_replace_key_check(self):
        dl = DependenciesList()
        dl.replace(object(), object())



"""Pylons requires that packages have a lib.base and lib.helpers 


So we've added on here so we can run tests in the context of the tg 
package itself, pylons will likely remove this restriction before 1.0 
and this module can then be removed. 

"""


import sys
import logging
from ..configuration.utils import coerce_config
from .._compat import reraise
from ..support.converters import asbool, asint
from .base import ApplicationWrapper

log = logging.getLogger(__name__)


class AbortTransaction(Exception):
    def __init__(self, response):
        self.response = response


class TransactionApplicationWrapper(ApplicationWrapper):
    """Wraps the whole application in zope.transaction transaction manager and
    rollbacks transaction in case of crashes.

    Supported options which can be provided by config are:

        - ``tm.enabled``: Wenever the transaction manager is enabled or not.
        - ``tm.attempts``: Number of times the transaction should be retried if
          it fails (no retry by default)
        - ``tm.commit_veto``: A function that will be called for every transaction to check
          if it should abort transaction or let it go. Function signature should be:
          ``function(environ, status_code, headers) -> bool``.

    """
    def __init__(self, handler, config):
        super(TransactionApplicationWrapper, self).__init__(handler, config)

        options = {
            'enabled': False,
            'attempts': 1,
            'commit_veto': None
        }
        options.update(coerce_config(config, 'tm.',  {
            'enabled': asbool,
            'attempts': asint,
        }))

        self.enabled = options['enabled']
        self.attempts = options['attempts']
        self.commit_veto = options['commit_veto']
        self.manager = None

        if self.enabled:
            try:
                import transaction
                self.manager = transaction.manager
            except ImportError:  # pragma: no cover
                log.exception('Unable to Enable Transaction Manager')
                self.enabled = False

        log.debug('TransactionManager enabled: %s -> %s',
                  self.enabled, options)

    @property
    def injected(self):
        return self.enabled

    def __call__(self, controller, environ, context):
        if 'repoze.tm.active' in environ:  # pragma: no cover
            # Skip transaction manager if repoze.tm2 is enabled
            return self.next_handler(controller, environ, context)

        transaction_manager = self.manager
        total_attempts = self.attempts
        commit_veto = self.commit_veto

        attempts_left = total_attempts
        while attempts_left:
            attempts_left -= 1

            log.debug('Attempts Left %d (%d total)', attempts_left, total_attempts)
            txn = transaction_manager.begin()
            try:
                response = self.next_handler(controller, environ, context)
                if txn.isDoomed():
                    log.debug('Transaction doomed')
                    raise AbortTransaction(response)

                if commit_veto is not None:
                    veto = commit_veto(environ, response.status, response.headerlist)
                    if veto:
                        log.debug('Transaction vetoed')
                        raise AbortTransaction(response)

                txn.commit()
                log.debug('Transaction committed!')
                return response
            except AbortTransaction as e:
                txn.abort()
                return e.response
            except:
                exc_info = sys.exc_info()
                log.debug('Error while running request, aborting transaction')
                try:
                    can_retry = transaction_manager._retryable(*exc_info[:-1])
                    txn.abort()
                    if (attempts_left <= 0) or (not can_retry):
                        reraise(*exc_info)
                finally:
                    del exc_info



# -*- coding: utf-8 -*-
from functools import wraps


def unless(func, check=None):
    """Wraps ``func`` ensuring it returns a value different from ``check``.

    A new function that calls ``func`` and checks its value is returned.
    In case func returns a value equal to check a ``ValueError`` is
    raised.

    A common usage pattern is to join this with  :class:`.Convert`
    to fail validation when querying objects from the database
    if they do not exist::

        Convert(unless(DBSession.query(User).get))

    """
    @wraps(func)
    def wrapper(*args, **kwargs):
        v = func(*args, **kwargs)
        if v == check:
            raise ValueError('{} == {}'.format(v, check))
        return v
    return wrapper




"""A fake application's model objects"""

from datetime import datetime

from zope.sqlalchemy import ZopeTransactionExtension
from sqlalchemy import Table, ForeignKey, Column
from sqlalchemy.orm import scoped_session, sessionmaker, relation, backref, \
                           synonym
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.types import String, Unicode, UnicodeText, Integer, DateTime, \
                             Boolean, Float


# Global session manager.  DBSession() returns the session object
# appropriate for the current web request.
maker = sessionmaker(autoflush=True, autocommit=False,
                     extension=ZopeTransactionExtension())
DBSession = scoped_session(maker)

# By default, the data model is defined with SQLAlchemy's declarative
# extension, but if you need more control, you can switch to the traditional
# method.
DeclarativeBase = declarative_base()

# Global metadata.
# The default metadata is the one from the declarative base.
metadata = DeclarativeBase.metadata

def init_model(engine):
    """Call me before using any of the tables or classes in the model."""
    DBSession.configure(bind=engine)


class Group(DeclarativeBase):
    """An ultra-simple group definition.
    """
    __tablename__ = 'tg_group'
    
    group_id = Column(Integer, autoincrement=True, primary_key=True)
    
    group_name = Column(Unicode(16), unique=True)
    
    display_name = Column(Unicode(255))
    
    created = Column(DateTime, default=datetime.now)
    
    def __repr__(self):
        return '<Group: name=%s>' % self.group_name



# -*- coding: utf-8 -*-
"""
Utilities for TurboGears hooks management.

Provides a consistent API to register and execute hooks.

"""
import atexit
import warnings
from .utils import TGConfigError
from .milestones import config_ready, renderers_ready, environment_loaded
from ..decorators import Decoration
from .._compat import default_im_func
from .app_config import config as tg_config


from logging import getLogger
log = getLogger(__name__)


class HooksNamespace(object):
    """Manages hooks registrations and notifications"""
    def __init__(self):
        self._hooks = dict()
        atexit.register(self._atexit)

    def _atexit(self):
        for func in self._hooks.get('shutdown', tuple()):
            func()

    def _call_handler(self, hook_name, trap_exceptions, func, args, kwargs):
        try:
            return func(*args, **kwargs)
        except:
            if trap_exceptions is True:
                log.exception('Trapped Exception while handling %s -> %s', hook_name, func)
            else:
                raise

    def register(self, hook_name, func, controller=None):
        """Registers a TurboGears hook.

        Given an hook name and a function it registers the provided
        function for that role. For a complete list of hooks
        provided by default have a look at :ref:`hooks_and_events`.

        It permits to register hooks both application wide
        or for specific controllers::

            tg.hooks.register('before_render', hook_func, controller=RootController.index)
            tg.hooks.register('startup', startup_function)

        """
        if hook_name in ('startup', 'shutdown') and controller is not None:
            raise TGConfigError('Startup and Shutdown hooks cannot be registered on controllers')

        if hook_name == 'controller_wrapper':
            raise TGConfigError('tg.hooks.wrap_controller must be used to register wrappers')

        if controller is None:
            config_ready.register(_ApplicationHookRegistration(self, hook_name, func))
        else:
            controller = default_im_func(controller)
            renderers_ready.register(_ControllerHookRegistration(controller, hook_name, func))

    def disconnect(self, hook_name, func, controller=None):
        """Disconnect an hook.

        The registered function is removed from the hook notification list.
        """
        if controller is None:
            registrations = self._hooks.get(hook_name, [])
        else:
            deco = Decoration.get_decoration(controller)
            registrations = deco.hooks.get(hook_name, [])

        try:
            registrations.remove(func)
        except ValueError:
            pass

    def notify(self, hook_name, args=None, kwargs=None, controller=None,
               context_config=None, trap_exceptions=False):
        """Notifies a TurboGears hook.

        Each function registered for the given hook will be executed,
        ``args`` and ``kwargs`` will be passed to the registered functions
        as arguments.

        It permits to notify both application hooks::

            tg.hooks.notify('custom_global_hook')

        Or controller hooks::

            tg.hooks.notify('before_render', args=(remainder, params, output),
                            controller=RootController.index)

        """
        args = args or []
        kwargs = kwargs or {}

        try:
            syswide_hooks = self._hooks[hook_name]
        except KeyError:  # pragma: no cover
            pass
        else:
            for func in syswide_hooks:
                self._call_handler(hook_name, trap_exceptions, func, args, kwargs)

        if controller is not None:
            controller = default_im_func(controller)
            deco = Decoration.get_decoration(controller)
            for func in deco.hooks.get(hook_name, []):
                self._call_handler(hook_name, trap_exceptions, func, args, kwargs)

    def notify_with_value(self, hook_name, value, controller=None, context_config=None):
        """Notifies a TurboGears hook which is expected to return a value.

        hooks with values are expected to accept an input value an return
        a replacement for it. Each registered function will receive as input
        the value returned by the previous function in chain.

        The resulting value will be returned by the ``notify_with_value``
        call itself::

            app = tg.hooks.notify_with_value('before_config', app)

        """
        try:
            syswide_hooks = self._hooks[hook_name]
        except KeyError:  # pragma: no cover
            pass
        else:
            for func in syswide_hooks:
                value = func(value)

        if controller is not None:
            controller = default_im_func(controller)
            deco = Decoration.get_decoration(controller)
            for func in deco.hooks[hook_name]:
                value = func(value)

        return value


class _ApplicationHookRegistration(object):
    def __init__(self, hooks_namespace, hook_name, func):
        self.hook_name = hook_name
        self.func = func
        self.hooks_namespace = hooks_namespace

    def __call__(self):
        log.debug("Registering %s for application wide hook %s",
                  self.func, self.hook_name)

        if self.hook_name == 'controller_wrapper':
            warnings.warn('controller wrappers should be registered on '
                          'AppConfig using AppConfig.register_controller_wrapper',
                          DeprecationWarning, stacklevel=3)

            config = tg_config._current_obj()
            config['controller_wrappers'].append(self.func)
        else:
            hooks = self.hooks_namespace._hooks
            hooks.setdefault(self.hook_name, []).append(self.func)


class _ControllerHookRegistration(object):
    def __init__(self, controller, hook_name, func):
        self.controller = controller
        self.hook_name = hook_name
        self.func = func

    def __call__(self):
        log.debug("Registering %s for hook %s on controller %s",
                  self.func, self.hook_name, self.controller)

        if self.hook_name == 'controller_wrapper':
            warnings.warn('controller wrappers should be registered on '
                          'AppConfig using AppConfig.register_controller_wrapper',
                          DeprecationWarning, stacklevel=3)

            deco = Decoration.get_decoration(self.controller)
            deco._register_controller_wrapper(self.func)
        else:
            deco = Decoration.get_decoration(self.controller)
            deco._register_hook(self.hook_name, self.func)


class _TGGlobalHooksNamespace(HooksNamespace):
    def wrap_controller(self, func, controller=None):
        """Registers a TurboGears controller wrapper.

        Controller Wrappers are much like a **decorator** applied to
        every controller.
        They receive :class:`tg.configuration.AppConfig` instance
        as an argument and the next handler in chain and are expected
        to return a new handler that performs whatever it requires
        and then calls the next handler.

        A simple example for a controller wrapper is a simple logging wrapper::

            def controller_wrapper(app_config, caller):
                def call(*args, **kw):
                    try:
                        print 'Before handler!'
                        return caller(*args, **kw)
                    finally:
                        print 'After Handler!'
                return call

            tg.hooks.wrap_controller(controller_wrapper)

        It is also possible to register wrappers for a specific controller::

            tg.hooks.wrap_controller(controller_wrapper, controller=RootController.index)

        """
        if environment_loaded.reached:
            raise TGConfigError('Controller wrappers can be registered only at '
                                'configuration time.')

        if controller is None:
            environment_loaded.register(_ApplicationHookRegistration(self,
                                                                     'controller_wrapper',
                                                                     func))
        else:
            controller = default_im_func(controller)
            registration = _ControllerHookRegistration(controller, 'controller_wrapper', func)
            renderers_ready.register(registration)

hooks = _TGGlobalHooksNamespace()



"""
Testing for TG2 Configuration
"""
from nose.tools import raises
from webtest import TestApp

import tg
from tg.render import MissingRendererError, _get_tg_vars
from tests.base import setup_session_dir, teardown_session_dir

from tg.configuration import AppConfig
from mako.exceptions import TemplateLookupException
from tg.util.webtest import test_context


def setup():
    setup_session_dir()
def teardown():
    teardown_session_dir()

class FakePackage:
    __name__ = 'tests'
    __file__ = __file__

    class lib:
        class app_globals:
            class Globals:
                pass

@raises(MissingRendererError)
def test_render_missing_renderer():
    conf = AppConfig(minimal=True)
    app = conf.make_wsgi_app()

    tg.render_template({}, 'gensh')


def test_render_default():
    conf = AppConfig(minimal=True)
    conf.default_renderer = 'json'
    app = conf.make_wsgi_app()

    res = tg.render_template({'value': 'value'})
    assert 'value": "value' in res


def test_jinja_lookup_nonexisting_template():
    conf = AppConfig(minimal=True)
    conf.use_dotted_templatenames = True
    conf.renderers.append('jinja')
    conf.package = FakePackage()
    app = conf.make_wsgi_app()

    from jinja2 import TemplateNotFound
    try:
        render_jinja = conf.render_functions['jinja']
        render_jinja('tg.this_template_does_not_exists',
                     {'app_globals':tg.config['tg.app_globals']})
        assert False
    except TemplateNotFound:
        pass


class TestKajikiSupport(object):
    def setup(self):
        conf = AppConfig(minimal=True)
        conf.use_dotted_templatenames = True
        conf.renderers.append('kajiki')
        conf.package = FakePackage()
        self.conf = conf
        self.app = TestApp(conf.make_wsgi_app())
        self.render = self.conf.render_functions['kajiki']

    def test_template_found(self):
        with test_context(self.app):
            res = self.render('tests.test_stack.rendering.templates.kajiki_i18n', {})
        assert 'Your application is now running' in res

    def test_dotted_template_not_found(self):
        try:
            with test_context(self.app):
                res = self.render('tests.test_stack.rendering.templates.this_doesnt_exists', {})
        except IOError as e:
            assert 'this_doesnt_exists.xhtml not found' in str(e)
        else:
            raise AssertionError('Should have raised IOError')

    def test_filename_template_not_found(self):
        try:
            with test_context(self.app):
                res = self.render('this_doesnt_exists/this_doesnt_exists.xhtml', {})
        except IOError as e:
            assert 'this_doesnt_exists.xhtml not found in template paths' in str(e)
        else:
            raise AssertionError('Should have raised IOError')


class TestMakoLookup(object):
    def setup(self):
        conf = AppConfig(minimal=True)
        conf.use_dotted_templatenames = True
        conf.renderers.append('mako')
        conf.package = FakePackage()
        self.conf = conf
        self.app = conf.make_wsgi_app()

    def test_adjust_uri(self):
        render_mako = self.conf.render_functions['mako']
        mlookup = render_mako.dotted_loader

        assert mlookup.adjust_uri('this_template_should_pass_unaltered', None) == 'this_template_should_pass_unaltered'

        dotted_test = mlookup.adjust_uri('tests.test_stack.rendering.templates.mako_inherits_local', None)
        assert dotted_test.endswith('tests/test_stack/rendering/templates/mako_inherits_local.mak')

        dotted_test = mlookup.adjust_uri('local:test_stack.rendering.templates.mako_inherits_local', None)
        assert dotted_test.endswith('tests/test_stack/rendering/templates/mako_inherits_local.mak')

    def test_local_lookup(self):
        render_mako = self.conf.render_functions['mako']
        res = render_mako('tests.test_stack.rendering.templates.mako_inherits_local',
                          {'app_globals':tg.config['tg.app_globals']})
        assert 'inherited mako page' in res

    def test_passthrough_text_literal__check(self):
        from mako.template import Template
        t = Template('Hi')

        render_mako = self.conf.render_functions['mako']
        mlookup = render_mako.dotted_loader
        mlookup.template_cache['hi_template'] = t
        assert mlookup.get_template('hi_template') is t

    @raises(TemplateLookupException)
    def test__check_not_existing_anymore(self):
        from mako.template import Template
        t = Template('Hi', filename='deleted_template.mak')

        render_mako = self.conf.render_functions['mako']
        mlookup = render_mako.dotted_loader
        mlookup.template_cache['deleted_template'] = t
        mlookup.get_template('deleted_template')

    @raises(IOError)
    def test_never_existed(self):
        render_mako = self.conf.render_functions['mako']
        mlookup = render_mako.dotted_loader

        mlookup.get_template('deleted_template')

    def test__check_should_reload_on_cache_expire(self):
        render_mako = self.conf.render_functions['mako']
        mlookup = render_mako.dotted_loader

        template_path = mlookup.adjust_uri('tests.test_stack.rendering.templates.mako_inherits_local', None)
        t = mlookup.get_template(template_path) #cache the template
        t.output_encoding = 'FAKE_ENCODING'

        t = mlookup.get_template(template_path)
        assert t.output_encoding == 'FAKE_ENCODING'

        import os, stat
        def fake_os_stat(o):
            return {stat.ST_MTIME:t.module._modified_time+1}

        old_stat = os.stat
        os.stat = fake_os_stat
        try:
            t = mlookup.get_template(template_path)
            #if the template got reloaded should not have our fake encoding anymore
            assert t.output_encoding != 'FAKE_ENCODING'
        finally:
            os.stat = old_stat

    def test__check_should_not_reload_when_disabled(self):
        render_mako = self.conf.render_functions['mako']
        mlookup = render_mako.dotted_loader
        mlookup.auto_reload = False

        template_path = mlookup.adjust_uri('tests.test_stack.rendering.templates.mako_inherits_local', None)
        t = mlookup.get_template(template_path) #cache the template
        t.output_encoding = 'FAKE_ENCODING'

        t = mlookup.get_template(template_path)
        assert t.output_encoding == 'FAKE_ENCODING'

        import os, stat
        def fake_os_stat(o):
            return {stat.ST_MTIME:t.module._modified_time+1}

        old_stat = os.stat
        os.stat = fake_os_stat
        try:
            t = mlookup.get_template(template_path)
            assert t.output_encoding == 'FAKE_ENCODING'
        finally:
            os.stat = old_stat

    def test_fallback_validation_context_in_templates(self):
        with test_context(None, '/'):
            vars = _get_tg_vars()
            assert vars.tg.errors == {}, vars.tg
            assert vars.tg.inputs == {}, vars.tg


import tg
import random, logging

try:
    from sqlalchemy.orm import Session
except ImportError: #pragma: no cover
    class Session(object):
        """SQLAlchemy Session"""

log = logging.getLogger(__name__)

class BalancedSession(Session):
    _force_engine = None

    def get_bind(self, mapper=None, clause=None):
        config = tg.config._current_obj()

        engines = config.get('balanced_engines')
        if not engines:
            log.debug('Balancing disabled, using master')
            return config['tg.app_globals'].sa_engine

        forced_engine = self._force_engine
        if not forced_engine:
            try:
                forced_engine = tg.request._tg_force_sqla_engine
            except TypeError:
                forced_engine = 'master'
            except AttributeError:
                pass

        if forced_engine:
            log.debug('Forced engine: %s', forced_engine)
            return engines['all'][forced_engine]
        elif self._flushing:
            log.debug('Choose engine: master')
            return engines['master']
        else:
            choosen_slave = random.choice(list(engines['slaves'].keys()))
            log.debug('Choose engine: %s', choosen_slave)
            return engines['slaves'][choosen_slave]

    def using_engine(self, engine_name):
        return UsingEngineContext(engine_name, self)

class UsingEngineContext(object):
    def __init__(self, engine_name, DBSession=None):
        self.engine_name = engine_name
        if not DBSession:
            DBSession = tg.config['DBSession']()
        self.session = DBSession
        self.past_engine = self.session._force_engine

    def __enter__(self):
        self.session._force_engine = self.engine_name
        return self.session

    def __exit__(self, exc_type, exc_value, traceback):
        self.session._force_engine = self.past_engine

def force_request_engine(engine_name):
    tg.request._tg_force_sqla_engine = engine_name



from ..jsonify import JSONEncoder
from ..jsonify import encode as json_encode


_script_json_encoder = JSONEncoder(isodates=True, allow_lists=True)


def script_json_encode(obj, encoder=_script_json_encoder, **kwargs):
    """Works exactly like :func:`tg.jsonify.encode` but is safe
    for use in ``<script>`` tags.

    The following characters are escaped in strings:
        -   ``<``
        -   ``>``
        -   ``&``
        -   ``'``

    This makes it safe to embed such strings in any place in HTML with the
    notable exception of double quoted attributes.  In that case single
    quote your attributes or HTML escape it in addition.
    """
    rv = json_encode(obj, encoder=encoder, **kwargs) \
        .replace('<', '\\u003c') \
        .replace('>', '\\u003e') \
        .replace('&', '\\u0026') \
        .replace("'", '\\u0027')

    return rv





from webtest import TestApp
from tg.support.middlewares import StatusCodeRedirect
from tg.support.middlewares import DBSessionRemoverMiddleware
from tg.support.middlewares import MingSessionRemoverMiddleware


def FakeApp(environ, start_response):
    if environ['PATH_INFO'].startswith('/error'):
        start_response('403 Forbidden', [])
    else:
        start_response('200 Success', [])

    if environ['PATH_INFO'] == '/error/document':
        yield b'ERROR!!!'
    else:
        yield b'HI'
        yield b'MORE'


class TestStatusCodeRedirectMiddleware(object):
    def setup(self):
        self.app = TestApp(StatusCodeRedirect(FakeApp, [403]))

    def test_error_redirection(self):
        r = self.app.get('/error_test', status=403)
        assert 'ERROR!!!' in r, r

    def test_success_passthrough(self):
        r = self.app.get('/success_test')
        assert 'HI' in r, r


class FakeDBSession(object):
    removed = False

    def remove(self):
        self.removed = True

    def close_all(self):
        self.remove()


class FakeAppWithClose(object):
    closed = False
    step = 0

    def __call__(self, environ, start_response):
        start_response('200 Success', [])

        if environ['PATH_INFO'] == '/crash':
            raise Exception('crashed')

        return self

    def __iter__(self):
        return self

    def next(self):
        self.step += 1

        if self.step > 3:
            raise StopIteration()

        return str(self.step)

    def close(self):
        self.closed = True

    def __repr__(self):
        return '%s - %s' % (self.step, self.closed)


class TestDBSessionRemoverMiddleware(object):
    def setup(self):
        self.app_with_close = FakeAppWithClose()
        self.session = FakeDBSession()
        self.app = TestApp(DBSessionRemoverMiddleware(self.session, self.app_with_close))

    def test_close_is_called(self):
        r = self.app.get('/nonerror')
        assert self.app_with_close.closed == True, self.app_with_close

    def test_session_is_removed(self):
        r = self.app.get('/nonerror')
        assert self.session.removed == True, self.app_with_close

    def test_session_is_removed_on_crash(self):
        try:
            r = self.app.get('/crash')
        except:
            pass

        assert self.session.removed == True, self.app_with_close


class TestMingSessionRemoverMiddlewaree(object):
    def setup(self):
        self.app_with_close = FakeAppWithClose()
        self.session = FakeDBSession()
        self.app = TestApp(MingSessionRemoverMiddleware(self.session, self.app_with_close))

    def test_close_is_called(self):
        r = self.app.get('/nonerror')
        assert self.app_with_close.closed == True, self.app_with_close

    def test_session_is_removed(self):
        r = self.app.get('/nonerror')
        assert self.session.removed == True, self.app_with_close

    def test_session_is_removed_on_crash(self):
        try:
            r = self.app.get('/crash')
        except:
            pass

        assert self.session.removed == True, self.app_with_close



# -*- coding: utf-8 -*-
from functools import partial
from nose.tools import raises
from nose import SkipTest
from crank.util import get_params_with_argspec

import tg
import tests
from json import loads
import datetime

from tg.controllers import TGController, DecoratedController, abort
from tg.controllers.util import validation_errors_response
from tg.decorators import expose, validate, before_render, before_call, Decoration
from tests.base import (TestWSGIController, data_dir,
    make_app, setup_session_dir, teardown_session_dir)

from tg._compat import PY3, unicode_text, u_, default_im_func
from tg.validation import TGValidationError, validation_errors, _ValidationStatus, Convert
from tg.i18n import lazy_ugettext as l_
from formencode import validators, Schema

import tw2.core as tw2c
import tw2.forms as tw2f


class MovieForm(tw2f.TableForm):
    title = tw2f.TextField(validator=tw2c.Required)
    year = tw2f.TextField(size=4, validator=tw2c.IntValidator)
movie_form = MovieForm(action='save_movie')

class Pwd(Schema):
    pwd1 = validators.String(not_empty=True)
    pwd2 = validators.String(not_empty=True)
    chained_validators = [validators.FieldsMatch('pwd1', 'pwd2')]

class FormWithFieldSet(tw2f.TableForm):
    class fields1(tw2f.ListFieldSet):
        f1 = tw2f.TextField(validator=tw2c.Required)

    class fields2(tw2f.ListFieldSet):
        f2 = tw2f.TextField(validator=tw2c.IntValidator)

if not PY3:
    from tw.forms import TableForm, TextField
    from tw.api import WidgetsList

    class MyForm(TableForm):
        class fields(WidgetsList):
            """This WidgetsList is just a container."""
            title=TextField(validator = validators.NotEmpty())
            year = TextField(size=4, validator=validators.Int())
    myform = MyForm("my_form", action='create')
else:
    myform = None


def setup():
    setup_session_dir()

def teardown():
    teardown_session_dir()


class controller_based_validate(validate):
    def __init__(self, error_handler=None, *args, **kw):
        self.error_handler = error_handler
        self.needs_controller = True

        class Validators(object):
            def validate(self, controller, params, state):
                return params

        self.validators = Validators()

class ColonValidator(validators.FancyValidator):
    def validate_python(self, value, state):
        raise validators.Invalid('ERROR: Description', value, state)

class ColonLessGenericValidator(object):
    def validate(self, value, state=None):
        raise validators.Invalid('Unknown Error', value, {'_the_form':'Unknown Error'})

class ThrowAwayValidationIntentValidator(object):
    def validate(self, value, state=None):
        tg.request.validation.intent = None
        raise validators.Invalid('Unknown Error', value, {'_the_form':'Unknown Error'})

def error_handler_function(controller_instance, uid, num):
    return 'UID: %s' % uid


def ControllerWrapperForErrorHandler(caller):
    def call(*args, **kw):
        value = caller(*args, **kw)
        return value + 'X'
    return call


class ErrorHandlerCallable(object):
    def __call__(self, controller_instance, uid, num):
        return 'UID: %s' % uid

class BasicTGController(TGController):
    @expose()
    @validate(ColonLessGenericValidator())
    def validator_without_columns(self, **kw):
        return tg.request.validation['errors']['_the_form']

    @expose('json:')
    @validate(validators={"some_int": validators.Int()})
    def validated_int(self, some_int):
        assert isinstance(some_int, int)
        return dict(response=some_int)

    @expose('json:')
    @validate(validators={"a": validators.Int()})
    def validated_and_unvalidated(self, a, b):
        assert isinstance(a, int)
        assert isinstance(b, unicode_text)
        return dict(int=a, str=b)

    @expose()
    @controller_based_validate()
    def validate_controller_based_validator(self, *args, **kw):
        return 'ok'

    @expose('json:')
    @validate(validators={"a": validators.Int(), "someemail": validators.Email()})
    def two_validators(self, a=None, someemail=None, *args):
        errors = tg.request.validation['errors']
        values = tg.request.validation['values']
        return dict(a=a, someemail=someemail,
                errors=str(errors), values=str(values))

    @expose('json:')
    @validate(validators={"a": validators.Int()})
    def with_default_shadow(self, a, b=None ):
        """A default value should not cause the validated value to disappear"""
        assert isinstance( a, int ), type(a)
        return {
            'int': a,
        }

    @expose('json:')
    @validate(validators={"e": ColonValidator()})
    def error_with_colon(self, e):
        errors = tg.request.validation['errors']
        return dict(errors=str(errors))

    @expose('json:')
    @validate(validators={
        "a": validators.Int(),"b":validators.Int(),"c":validators.Int(),"d":validators.Int()
    })
    def with_default_shadow_long(self, a, b=None,c=None,d=None ):
        """A default value should not cause the validated value to disappear"""
        assert isinstance( a, int ), type(a)
        assert isinstance( b, int ), type(b)
        assert isinstance( c, int ), type(c)
        assert isinstance( d, int ), type(d)
        return {
            'int': [a,b,c,d],
        }

    @expose()
    def display_form(self, **kwargs):
        return str(myform.render(values=kwargs))

    @expose('json:')
    @validate(form=myform)
    def process_form(self, **kwargs):
        kwargs['errors'] = tg.request.validation['errors']
        return dict(kwargs)

    @expose('json:')
    @validate(form=myform, error_handler=process_form)
    def send_to_error_handler(self, **kwargs):
        kwargs['errors'] = tg.request.validation['errors']
        return dict(kwargs)

    @expose('json')
    def tw2form_error_handler(self, **kwargs):
        return dict(errors=tg.request.validation['errors'])

    @expose('json:')
    @validate(form=movie_form, error_handler=tw2form_error_handler)
    def send_tw2_to_error_handler(self, **kwargs):
        return 'passed validation'

    @expose()
    @validate({'param': tw2c.IntValidator()},
              error_handler=validation_errors_response)
    def tw2_dict_validation(self, **kwargs):
        return 'NO_ERROR'

    @expose()
    @validate({'param': validators.Int()},
              error_handler=validation_errors_response)
    def formencode_dict_validation(self, **kwargs):
        return 'NO_ERROR'

    @expose('text/plain')
    @validate(form=FormWithFieldSet, error_handler=tw2form_error_handler)
    def tw2_fieldset_submit(self, **kwargs):
        return 'passed validation'

    @expose()
    def set_lang(self, lang=None):
        tg.session['tg_lang'] = lang
        tg.session.save()
        return 'ok'

    @expose()
    @validate(validators=Pwd())
    def password(self, pwd1, pwd2):
        if tg.request.validation['errors']:
            return "There was an error"
        else:
            return "Password ok!"

    @expose('json:')
    @before_render(lambda rem,params,output:output.update({'GOT_ERROR':'HOOKED'}))
    def hooked_error_handler(self, *args, **kw):
        return dict(GOT_ERROR='MISSED HOOK')

    @expose()
    @validate({'v':validators.Int()}, error_handler=hooked_error_handler)
    def with_hooked_error_handler(self, *args, **kw):
        return dict(GOT_ERROR='NO ERROR')

    @expose('json')
    @validate({'v': validators.Int()})
    def check_tmpl_context_compatibility(self, *args, **kw):
        return dict(tmpl_errors=str(tg.tmpl_context.form_errors),
                    errors=str(tg.request.validation['errors']))

    @expose()
    def error_handler(self, *args, **kw):
        return 'ERROR HANDLER!'

    @expose('json:')
    @validate(validators={"some_int": validators.Int()},
              error_handler=error_handler)
    def validate_other_error_handler(self, some_int):
        return dict(response=some_int)

    def unexposed_error_handler(self, uid, **kw):
        return 'UID: %s' % uid

    @expose()
    @validate({'uid': validators.Int(),
               'num': validators.Int()},
              error_handler=unexposed_error_handler)
    def validate_unexposed(self, uid, num):
        return 'HUH'

    @expose()
    @validate({'num': validators.Int()},
              error_handler=partial(unexposed_error_handler,
                                    uid=5))
    def validate_partial(self, num):
        return 'HUH'

    @expose()
    @validate({'uid': tw2c.IntValidator(),
               'num': tw2c.IntValidator()},
              error_handler=error_handler_function)
    def validate_function(self, uid, num):
        return 'HUH'

    @expose()
    @validate({'uid': validators.Int(),
               'num': validators.Int()},
              error_handler=ErrorHandlerCallable())
    def validate_callable(self, uid, num):
        return 'HUH'

    @expose()
    @validate({'uid': validators.Int()},
              error_handler=ErrorHandlerCallable())
    @validate({'num': validators.Int()},
              error_handler=abort(412, error_handler=True))
    def validate_multi(self, uid, num):
        return str(uid+num)

    @expose()
    @validate({'uid': validators.Int()},
              error_handler=abort(412, error_handler=True))
    def abort_error_handler(self):
        return 'HUH'

    @expose()
    @validate({'uid': validators.Int()},
              error_handler=validation_errors_response)
    def validate_json_errors(self):
        return 'HUH'

    @expose()
    def validate_json_errors_complex_types(self, date):
        tg.request.validation.values = {'date': datetime.datetime.utcnow()}
        return validation_errors_response()

    @expose()
    @before_call(lambda remainder, params: params.setdefault('num', 5))
    def hooked_error_handler(self, uid, num):
        return 'UID: %s, NUM: %s' % (uid, num)

    @expose()
    @validate(ThrowAwayValidationIntentValidator(),
              error_handler=abort(412, error_handler=True))
    def throw_away_intent(self, uid):
        if tg.request.validation.exception:
            return 'ERROR'
        return 'UHU?'

    @expose()
    @validate(error_handler=hooked_error_handler)
    def passthrough_validation(self, uid):
        return str(uid)

    @expose()
    @validate({'uid': validators.Int()},
              error_handler=hooked_error_handler)
    def validate_hooked(self, uid):
        return 'HUH'

    # Decorate validate_hooked with a controller wrapper
    Decoration.get_decoration(hooked_error_handler)\
        ._register_controller_wrapper(ControllerWrapperForErrorHandler)

    @expose()
    def manually_handle_validation(self):
        # This is done to check that we don't break compatibility
        # with external modules that perform custom validation like tgext.socketio

        controller = self.__class__.validate_function
        args = (2, 'NaN')
        try:
            output = ''
            validate_params = get_params_with_argspec(controller, {}, args)
            params = DecoratedController._perform_validate(controller,
                                                           validate_params)
        except validation_errors as inv:
            handler, output = DecoratedController._handle_validation_errors(controller,
                                                                            args, {},
                                                                            inv, None)

        return output

    @expose(content_type='text/plain')
    @validate({
        'num': Convert(int, l_('This must be a number'))
    }, error_handler=validation_errors_response)
    def post_pow2(self, num=-1):
        return str(num*num)

    @expose(content_type='text/plain')
    @validate({
        'num': Convert(int, l_('This must be a number'), default=0)
    }, error_handler=validation_errors_response)
    def post_pow2_opt(self, num=-1):
        return str(num*num)

    @expose(content_type='text/plain')
    @validate({
        'num': Convert(int, u_('àèìòù'))
    }, error_handler=validation_errors_response)
    def unicode_error_pow(self, num=-1):
        return str(num*num)

    @expose(content_type='text/plain')
    @validate({
        'num': Convert(int, l_(u_('àèìòù')))
    }, error_handler=validation_errors_response)
    def lazy_unicode_error_pow(self, num=-1):
        return str(num * num)


class TestTGController(TestWSGIController):
    def setUp(self):
        TestWSGIController.setUp(self)
        tg.config.update({
            'paths': {'root': data_dir},
            'package': tests,
        })

        self.app = make_app(BasicTGController, config_options={
            'i18n.enabled': True
        })

    def test_basic_validation_and_jsonification(self):
        """Ensure you can pass in a dictionary of validators"""
        form_values = {"some_int": 22}
        resp = self.app.post('/validated_int', form_values)
        assert '{"response": 22}'in resp, resp

    def test_validation_other_error_handler(self):
        form_values = {"some_int": 'TEXT'}
        resp = self.app.post('/validate_other_error_handler', form_values)
        assert 'ERROR HANDLER!'in resp, resp

    def test_validator_without_columns(self):
        form_values = {"some_int": 22}
        resp = self.app.post('/validator_without_columns', form_values)
        assert 'Unknown Error' in resp, resp

    def test_for_other_params_after_validation(self):
        """Ensure that both validated and unvalidated data make it through"""
        form_values = {'a': 1, 'b': "string"}
        resp = self.app.post('/validated_and_unvalidated', form_values)
        assert '"int": 1' in resp
        assert '"str": "string"' in resp, resp

    def test_validation_shadowed_by_defaults( self ):
        """Catch regression on positional argument validation with defaults"""
        resp = self.app.post('/with_default_shadow/1?b=string')
        assert '"int": 1' in resp, resp

    def test_optional_shadowed_by_defaults( self ):
        """Catch regression on optional arguments being reverted to un-validated"""
        resp = self.app.post('/with_default_shadow_long/1?b=2&c=3&d=4')
        assert '"int": [1, 2, 3, 4]' in resp, resp

    @raises(AssertionError)
    def test_validation_fails_with_no_error_handler(self):
        form_values = {'a':'asdf', 'b':"string"}
        resp = self.app.post('/validated_and_unvalidated', form_values)

    def test_two_validators_errors(self):
        """Ensure that multiple validators are applied correctly"""
        form_values = {'a': '1', 'someemail': "guido@google.com"}
        resp = self.app.post('/two_validators', form_values)
        content = loads(resp.body.decode('utf-8'))
        assert content['a'] == 1

    def test_validation_errors(self):
        """Ensure that dict validation produces a full set of errors"""
        form_values = {'a': '1', 'someemail': "guido~google.com"}
        resp = self.app.post('/two_validators', form_values)
        content = loads(resp.body.decode('utf-8'))
        errors = content.get('errors', None)
        assert errors, 'There should have been at least one error'
        assert 'someemail' in errors, \
            'The email was invalid and should have been reported in the errors'

    def test_form_validation(self):
        """Check @validate's handing of ToscaWidget forms instances"""
        if PY3: raise SkipTest()

        form_values = {'title': 'Razer', 'year': "2007"}
        resp = self.app.post('/process_form', form_values)
        values = loads(resp.body.decode('utf-8'))
        assert values['year'] == 2007

    def test_error_with_colon(self):
        resp = self.app.post('/error_with_colon', {'e':"fakeparam"})
        assert 'Description' in str(resp.body), resp.body

    def test_form_render(self):
        """Test that myform renders properly"""
        if PY3: raise SkipTest()

        resp = self.app.post('/display_form')
        assert 'id="my_form_title.label"' in resp, resp
        assert 'class="fieldlabel required"' in resp, resp
        assert "Title" in resp, resp

    def test_form_validation_error(self):
        """Test form validation with error message"""
        if PY3: raise SkipTest()

        form_values = {'title': 'Razer', 'year': "t007"}
        resp = self.app.post('/process_form', form_values)
        values = loads(resp.body.decode('utf-8'))
        assert "Please enter an integer value" in values['errors']['year'], \
            'Error message not found: %r' % values['errors']

    def test_form_validation_redirect(self):
        """Test form validation error message with redirect"""
        if PY3: raise SkipTest()

        form_values = {'title': 'Razer', 'year': "t007"}
        resp = self.app.post('/send_to_error_handler', form_values)
        values = loads(resp.body.decode('utf-8'))
        assert "Please enter an integer value" in values['errors']['year'], \
            'Error message not found: %r' % values['errors']

    def test_tw2form_validation(self):
        form_values = {'title': 'Razer', 'year': "t007"}
        resp = self.app.post('/send_tw2_to_error_handler', form_values)
        values = loads(resp.body.decode('utf-8'))
        assert "Must be an integer" in values['errors']['year'],\
        'Error message not found: %r' % values['errors']

    def test_tw2dict_validation(self):
        resp = self.app.post('/tw2_dict_validation', {'param': "7"})
        assert 'NO_ERROR' in str(resp.body)

        resp = self.app.post('/tw2_dict_validation', {'param': "hello"}, status=412)
        assert 'Must be an integer' in str(resp.body)

    def test_formencode_dict_validation(self):
        resp = self.app.post('/formencode_dict_validation', {'param': "7"})
        assert 'NO_ERROR' in str(resp.body), resp

        resp = self.app.post('/formencode_dict_validation', {'param': "hello"}, status=412)
        assert 'Please enter an integer value' in str(resp.body), resp

    def test_form_validation_translation(self):
        if PY3: raise SkipTest()

        """Test translation of form validation error messages"""
        form_values = {'title': 'Razer', 'year': "t007"}
        # check with language set in request header
        resp = self.app.post('/process_form', form_values,
            headers={'Accept-Language': 'de,ru,it'})
        values = loads(resp.body.decode('utf-8'))
        assert "Bitte eine ganze Zahl eingeben" in values['errors']['year'], \
            'No German error message: %r' % values['errors']
        resp = self.app.post('/process_form', form_values,
            headers={'Accept-Language': 'ru,de,it'})
        values = loads(resp.body.decode('utf-8'))
        assert u_("Введите числовое значение") in values['errors']['year'], \
            'No Russian error message: %r' % values['errors']
        # check with language set in session
        self.app.post('/set_lang/de')
        resp = self.app.post('/process_form', form_values,
            headers={'Accept-Language': 'ru,it'})
        values = loads(resp.body.decode('utf-8'))
        assert "Bitte eine ganze Zahl eingeben" in values['errors']['year'], \
            'No German error message: %r' % values['errors']

    def test_form_validation_error(self):
        """Test schema validation"""
        form_values = {'pwd1': 'me', 'pwd2': 'you'}
        resp = self.app.post('/password', form_values)
        assert "There was an error" in resp, resp
        form_values = {'pwd1': 'you', 'pwd2': 'you'}
        resp = self.app.post('/password', form_values)
        assert "Password ok!" in resp, resp

    def test_controller_based_validator(self):
        """Test controller based validation"""
        resp = self.app.post('/validate_controller_based_validator')
        assert 'ok' in resp, resp

    def test_hook_after_validation_error(self):
        resp = self.app.post('/with_hooked_error_handler?v=a')
        assert 'HOOKED' in resp, resp

    def test_check_tmpl_context_compatibility(self):
        resp = self.app.post('/check_tmpl_context_compatibility?v=a')
        resp = resp.json
        assert resp['errors'] == resp['tmpl_errors'], resp

    def test_validation_error_has_message(self):
        e = TGValidationError('This is a validation error')
        assert str(e) == 'This is a validation error'

    def test_tw2_fieldset(self):
        form_values = {'fields1:f1': 'Razer', 'fields2:f2': "t007"}
        resp = self.app.post('/tw2_fieldset_submit', form_values)
        values = loads(resp.body.decode('utf-8'))

        assert "Must be an integer" in values['errors'].get('fields2:f2', ''),\
        'Error message not found: %r' % values['errors']

    def test_validate_partial(self):
        resp = self.app.post('/validate_partial', {'num': 'NaN'})
        assert resp.text == 'UID: 5', resp

    def test_validate_unexposed(self):
        resp = self.app.post('/validate_unexposed', {'uid': 2,
                                                     'num': 'NaN'})
        assert resp.text == 'UID: 2', resp

    def test_validate_function(self):
        resp = self.app.post('/validate_function', {'uid': 2,
                                                    'num': 'NaN'})
        assert resp.text == 'UID: 2', resp

    def test_validate_callable(self):
        resp = self.app.post('/validate_callable', {'uid': 2,
                                                    'num': 'NaN'})
        assert resp.text == 'UID: 2', resp

    def test_validate_multi(self):
        resp = self.app.post('/validate_multi', {'uid': 'NaN', 'num': 2})
        assert resp.text == 'UID: NaN', resp

        resp = self.app.post('/validate_multi', {'uid': 2, 'num': 'NaN'}, status=412)
        assert resp.status.startswith('412')

        resp = self.app.post('/validate_multi', {'uid': 2, 'num': 2})
        assert resp.text == '4', resp

        resp = self.app.post('/validate_multi', {'uid': 'NaN', 'num': 'NaN'})
        assert resp.text == 'UID: NaN', resp

    def test_validate_hooked(self):
        resp = self.app.post('/validate_hooked', {'uid': 'NaN'})
        assert resp.text == 'UID: NaN, NUM: 5X', resp

    def test_manually_handle_validation(self):
        # This is done to check that we don't break compatibility
        # with external modules that perform custom validation like tgext.socketio
        resp = self.app.post('/manually_handle_validation')
        assert resp.text == 'UID: 2', resp

    def test_abort_error_handler(self):
        resp = self.app.post('/abort_error_handler', {'uid': 'NaN'}, status=412)
        assert resp.status.startswith('412')

    def test_json_error_handler(self):
        resp = self.app.post('/validate_json_errors', {'uid': 'NaN'}, status=412)
        assert resp.json['errors']['uid'] == 'Please enter an integer value'

    def test_json_error_handler_complex_type(self):
        resp = self.app.post('/validate_json_errors_complex_types', {'date': '2014-01-01'},
                             status=412)
        assert resp.json['values']['date'] == '2014-01-01', resp

    def test_throw_way_validation_intent(self):
        # This should actually never happen.
        # It requires the validator to mess up with TG internals, it's just to provide full coverage.
        resp = self.app.post('/throw_away_intent', {'uid': 5})
        assert resp.text == 'ERROR', resp

    def test_passthrough_validation(self):
        # This should actually never happen.
        # It requires the validator to mess up with TG internals, it's just to provide full coverage.
        resp = self.app.post('/passthrough_validation', {'uid': 5})
        assert resp.text == '5', resp

    def test_ValidationStatus_asdict(self):
        vs = _ValidationStatus()
        assert vs['errors'] is vs.errors
        assert vs['values'] is vs.values
        assert vs['error_handler'] is vs.error_handler

        try:
            vs['this_does_not_exists']
        except KeyError:
            pass
        else:
            assert False, 'Should have raised KeyError'

    def test_backward_compatibility_decorator(self):
        deco = Decoration.get_decoration(BasicTGController.two_validators)

        validated_params = sorted(list(deco.validation.validators.keys()))
        assert validated_params == ["a", "someemail"], validated_params

        deco = Decoration.get_decoration(BasicTGController.tw2form_error_handler)
        assert deco.validation is None, deco.validation

    def test_convert_validation(self):
        resp = self.app.post('/post_pow2', {'num': '5'})
        assert resp.text == '25', resp

    def test_convert_validation_fail(self):
        resp = self.app.post('/post_pow2', {'num': 'HELLO'}, status=412)
        assert 'This must be a number' in resp.json['errors']['num']

    def test_convert_validation_missing(self):
        resp = self.app.post('/post_pow2', {'num': ''}, status=412)
        assert 'This must be a number' in resp.json['errors']['num']

        resp = self.app.post('/post_pow2', status=412)
        assert 'This must be a number' in resp.json['errors']['num']

    def test_convert_validation_optional(self):
        resp = self.app.post('/post_pow2_opt', {'num': 'HELLO'}, status=412)
        assert 'This must be a number' in resp.json['errors']['num']

        resp = self.app.post('/post_pow2_opt', {'num': '5'})
        assert resp.text == '25', resp

        resp = self.app.post('/post_pow2_opt', {'num': ''})
        assert resp.text == '0', resp

        resp = self.app.post('/post_pow2_opt')
        assert resp.text == '0', resp

    def test_validation_errors_unicode(self):
        resp = self.app.post('/unicode_error_pow', {'num': 'NOT_A_NUMBER'}, status=412)
        assert resp.json['errors']['num'] == u_('àèìòù'), resp.json

    def test_validation_errors_lazy_unicode(self):
        resp = self.app.post('/lazy_unicode_error_pow', {'num': 'NOT_A_NUMBER'}, status=412)
        assert resp.json['errors']['num'] == u_('àèìòù'), resp.json


"""
Testing for TG2 Configuration
"""
from nose import SkipTest
from nose.tools import eq_, raises
import atexit, sys, os
from datetime import datetime
from sqlalchemy.orm import scoped_session
from sqlalchemy.orm import sessionmaker
from sqlalchemy.engine import Engine
from ming import Session
from ming.orm import ThreadLocalORMSession
from tg.configuration.hooks import _TGGlobalHooksNamespace

from tg.appwrappers.errorpage import ErrorPageApplicationWrapper
from tg.appwrappers.mingflush import MingApplicationWrapper
from tg.appwrappers.transaction_manager import TransactionApplicationWrapper

from tg.util import Bunch
from tg.configuration import AppConfig, config
from tg.configuration.app_config import TGConfigError, defaults as app_config_defaults
from tg.configuration.auth import _AuthenticationForgerPlugin
from tg.configuration.auth.metadata import _AuthMetadataAuthenticator
from tg.configuration.utils import coerce_config, coerce_options
from tg.configuration import milestones
from tg.support.converters import asint, asbool

import tg.i18n
from tg import TGController, expose, response, request, abort
from tests.base import setup_session_dir, teardown_session_dir
from webtest import TestApp
from tg.renderers.base import RendererFactory

from tg.wsgiapp import TGApp
from tg._compat import PY3

def setup():
    milestones._reset_all()
    setup_session_dir()
def teardown():
    milestones._reset_all()
    teardown_session_dir()


class PackageWithModel:
    __name__ = 'tests'
    __file__ = __file__

    def __init__(self):
        self.model = self.ModelClass()
        self.model.DBSession = self.model.FakeDBSession()

    class ModelClass:
        class FakeDBSession:
            def remove(self):
                self.DBSESSION_REMOVED=True

        def init_model(self, engine):
            if isinstance(engine, Engine):
                # SQLA
                return self.DBSession
            else:
                # Ming
                return dict(ming=True)

    class lib:
        class app_globals:
            class Globals:
                pass
PackageWithModel.__name__ = 'tests'


class UncopiableList(list):
    """
    This is to test configuration methods that make a copy
    of a list to modify it, using this we can check how it has
    been modified
    """
    def __copy__(self):
        return self

class FakeTransaction:
    def get(self):
        return self

    def begin(self):
        self.aborted = False
        self.doomed = False
        return self

    def abort(self):
        self.aborted = True

    def commit(self):
        self.aborted = False

    def _retryable(self, *args):
        return True
    note = _retryable

    def isDoomed(self):
        return self.doomed

    def doom(self):
        self.doomed = True

from tg.configuration.auth import TGAuthMetadata
class ApplicationAuthMetadata(TGAuthMetadata):
    def get_user(self, identity, userid):
        return {'name':'None'}

class ApplicationAuthMetadataWithAuthentication(TGAuthMetadata):
    def authenticate(self, environ, identity):
        return 1
    def get_user(self, identity, userid):
        return {'name':'None'}

class AtExitTestException(Exception):
    pass

class TestPylonsConfigWrapper:

    def setup(self):
        self.config = config

    def test_create(self):
        pass

    def test_getitem(self):
        expected_keys = ['global_conf', 'use_sqlalchemy', 'package', 'tg.app_globals', 'call_on_shutdown']
        for key in expected_keys:
            self.config[key]

    @raises(KeyError)
    def test_getitem_bad(self):
        self.config['no_such_key']

    def test_setitem(self):
        self.config['no_such_key'] = 'something'

    def test_delattr(self):
        del self.config.use_sqlalchemy
        eq_(hasattr(self.config, 'use_sqlalchemy'), False)
        self.config.use_sqlalchemy = True

    @raises(AttributeError)
    def test_delattr_bad(self):
        del self.config.i_dont_exist

    def test_keys(self):
        k = self.config.keys()
        assert 'tg.app_globals' in k


def test_coerce_config():
    opts = {'ming.connection.max_pool_size': '5'}
    conf = coerce_config(opts, 'ming.connection.', {'max_pool_size':asint})
    assert conf['max_pool_size'] == 5
    assert opts['ming.connection.max_pool_size'] == '5'


def test_coerce_options():
    opts = {'connection': 'false'}
    conf = coerce_options(opts, {'connection': asbool})
    assert conf['connection'] is False
    assert opts['connection'] == 'false'


class TestAppConfig:
    def __init__(self):
        self.fake_package = PackageWithModel

    def setup(self):
        milestones._reset_all()

        self.config = AppConfig()
        # set up some required paths and config settings
        # FIXME: these seem to be needed so that
        # other tests don't suffer - but that's a nasty
        # side-effect. setup for those tests actually needs
        # fixing.
        self.config['package'] = self.fake_package
        self.config['paths']['root'] = 'test'
        self.config['paths']['controllers'] = 'test.controllers'
        self.config['paths']['static_files'] = "test"
        self.config["tg.app_globals"] = Bunch()
        self.config["use_sqlalchemy"] = False
        self.config["global_conf"] = Bunch()
        self.config["render_functions"] = Bunch()
        self.config['session.secret'] = 'some_secret'
        self.config._init_config({'cache_dir':'/tmp'}, {})


    def teardown(self):
        #This is here to avoid that other tests keep using the forced controller
        config.pop('tg.root_controller', None)
        milestones._reset_all()
        tg.hooks = _TGGlobalHooksNamespace()  # Reset hooks

    def test_reqlocal_configuration_dictionary(self):
        self.config['RANDOM_VALUE'] = 5
        conf = self.config._init_config({}, {})

        assert config['RANDOM_VALUE'] == 5
        assert len(config) == len(conf)

    def test_get_root(self):
        current_root_module = self.config['paths']['root']
        assert self.config._get_root_module() == 'tests.controllers.root', self.config._get_root_module()
        self.config['paths']['root'] = None
        assert self.config._get_root_module() == None, self.config._get_root_module()
        self.config['paths']['root'] = current_root_module

    def test_lang_can_be_changed_by_ini(self):
        conf = AppConfig(minimal=True)
        conf._init_config({'lang':'ru'}, {})
        assert config['lang'] == 'ru'

    def test_create_minimal_app(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        conf = AppConfig(minimal=True, root_controller=RootController())
        app = conf.make_wsgi_app()
        app = TestApp(app)
        assert 'HI!' in app.get('/test')

        #This is here to avoid that other tests keep using the forced controller
        config.pop('tg.root_controller')

    def test_minimal_app_with_sqlalchemy(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        DBSession = scoped_session(sessionmaker(autoflush=True, autocommit=False))
        def init_model(engine):
            DBSession.configure(bind=engine)

        conf = AppConfig(minimal=True, root_controller=RootController())

        conf['use_sqlalchemy'] = True
        conf['sqlalchemy.url'] = 'sqlite://'
        conf['model'] = Bunch(DBSession=DBSession,
                              init_model=init_model)

        app = conf.make_wsgi_app()
        app = TestApp(app)
        assert 'HI!' in app.get('/test')

    @raises(TGConfigError)
    def test_sqlalchemy_without_models(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['use_sqlalchemy'] = True
        conf['sqlalchemy.url'] = 'sqlite://'
        app = conf.make_wsgi_app()

    def test_minimal_app_with_ming(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        mainsession = Session()
        DBSession = ThreadLocalORMSession(mainsession)

        def init_model(engine):
            mainsession.bind = engine

        conf = AppConfig(minimal=True, root_controller=RootController())

        conf['use_ming'] = True
        conf['ming.url'] = 'mim://'
        conf['model'] = Bunch(init_model=init_model, DBSession=DBSession)

        app = conf.make_wsgi_app()
        app = TestApp(app)
        assert 'HI!' in app.get('/test')

    @raises(TGConfigError)
    def test_ming_without_models(self):
        class RootController(TGController):
            @expose()
            def test(self):

                return 'HI!'

        DBSession = scoped_session(sessionmaker(autoflush=True, autocommit=False))
        def init_model(engine):
            DBSession.configure(bind=engine)

        conf = AppConfig(minimal=True, root_controller=RootController())

        conf['use_ming'] = True
        conf['ming.url'] = 'mim://'
        app = conf.make_wsgi_app()

    def test_create(self):
        pass

    def test_setup_helpers_and_globals(self):
        self.config._setup_helpers_and_globals(self.config._init_config({}, {}))

    def test_setup_helpers_and_globals_custom_backward_compatible(self):
        def custom_helpers(conf):
            conf['helpers'] = 'YES!'

        object.__setattr__(self.config, '_setup_helpers_and_globals', custom_helpers)
        conf = self.config.make_load_environment()({}, {})
        assert conf['helpers'] == 'YES!', conf.get('helpers')

    def test_setup_sa_auth_backend(self):
        class ConfigWithSetupAuthBackend(self.config.__class__):
            called = []

            def setup_sa_auth_backend(self):
                self.called.append(True)

        conf = ConfigWithSetupAuthBackend()
        conf._setup_auth(conf._init_config({}, {}))

        assert len(ConfigWithSetupAuthBackend.called) >= 1

    def test_setup_sa_auth_custom_backward_compatible(self):
        class ConfigWithSetupAuthBackend(self.config.__class__):
            called = []

            def _setup_auth(self, app_config):
                self.called.append(True)

        conf = ConfigWithSetupAuthBackend()
        conf._setup_auth(conf._init_config({}, {}))

        assert len(ConfigWithSetupAuthBackend.called) >= 1

    def test_setup_jinja_without_package(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.renderers = ['jinja']
        app = conf.make_wsgi_app()

    def test_setup_sqlalchemy(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_sqlalchemy = True
        conf['sqlalchemy.url'] = 'sqlite://'
        app = conf.make_wsgi_app()
        app = TestApp(app)

        assert 'HI!' in app.get('/test')
        assert package.model.DBSession.DBSESSION_REMOVED

    def test_custom_transaction_manager(self):
        class CustomAppConfig(AppConfig):
            def _add_tm_middleware(self, config, app):
                self.did_perform_custom_tm = True
                return app

        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        package = PackageWithModel()
        conf = CustomAppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_sqlalchemy = True
        conf.use_transaction_manager = True
        conf['sqlalchemy.url'] = 'sqlite://'

        app = conf.make_wsgi_app()

        # Check the custom manager got configured
        assert conf.did_perform_custom_tm == True

        # The transaction manager wrapper should not have been enabled.
        tgapp = TGApp()
        wrapper = tgapp.wrapped_dispatch
        while wrapper != tgapp._dispatch:
            assert not isinstance(wrapper, TransactionApplicationWrapper)
            wrapper = wrapper.next_handler

    def test_sqlalchemy_commit_veto(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

            @expose()
            def crash(self):
                raise Exception('crash')

            @expose()
            def forbidden(self):
                response.status = 403
                return 'FORBIDDEN'

            @expose()
            def notfound(self):
                response.status = 404
                return 'NOTFOUND'

        def custom_commit_veto(environ, status, headers):
            if status.startswith('404'):
                return True
            return False

        fake_transaction = FakeTransaction()
        import transaction
        prev_transaction_manager = transaction.manager
        transaction.manager = fake_transaction

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_sqlalchemy = True
        conf['tm.enabled'] = True
        conf['tm.commit_veto'] = custom_commit_veto
        conf['sqlalchemy.url'] = 'sqlite://'

        app = conf.make_wsgi_app()
        app = TestApp(app)
        assert hasattr(conf, 'use_transaction_manager') is False

        app.get('/test')
        assert fake_transaction.aborted == False

        try:
            app.get('/crash')
        except:
            pass
        assert fake_transaction.aborted == True

        app.get('/forbidden', status=403)
        assert fake_transaction.aborted == False

        app.get('/notfound', status=404)
        assert fake_transaction.aborted == True

        transaction.manager = prev_transaction_manager

    def test_sqlalchemy_doom(self):
        fake_transaction = FakeTransaction()
        import transaction
        prev_transaction_manager = transaction.manager
        transaction.manager = fake_transaction

        class RootController(TGController):
            @expose()
            def test(self):
                fake_transaction.doom()
                return 'HI!'

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_sqlalchemy = True
        conf['tm.enabled'] = True
        conf['sqlalchemy.url'] = 'sqlite://'

        app = conf.make_wsgi_app()
        app = TestApp(app)
        assert hasattr(conf, 'use_transaction_manager') is False

        app.get('/test')
        assert fake_transaction.aborted == True

        transaction.manager = prev_transaction_manager

    def test_sqlalchemy_retry(self):
        fake_transaction = FakeTransaction()
        import transaction
        prev_transaction_manager = transaction.manager
        transaction.manager = fake_transaction

        from transaction.interfaces import TransientError

        class RootController(TGController):
            attempts = []

            @expose()
            def test(self):
                self.attempts.append(True)
                if len(self.attempts) == 3:
                    return 'HI!'
                raise TransientError()

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_sqlalchemy = True
        conf['tm.enabled'] = True
        conf['sqlalchemy.url'] = 'sqlite://'
        conf['tm.attempts'] = 3

        app = conf.make_wsgi_app()
        app = TestApp(app)
        assert hasattr(conf, 'use_transaction_manager') is False

        resp = app.get('/test')
        assert 'HI' in resp

        transaction.manager = prev_transaction_manager

    def test_old_sqlalchemy_commit_veto(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

            @expose()
            def crash(self):
                raise Exception('crash')

            @expose()
            def forbidden(self):
                response.status = 403
                return 'FORBIDDEN'

            @expose()
            def notfound(self):
                response.status = 404
                return 'NOTFOUND'

        def custom_commit_veto(environ, status, headers):
            if status.startswith('404'):
                return True
            return False

        fake_transaction = FakeTransaction()
        import transaction
        prev_transaction_manager = transaction.manager
        transaction.manager = fake_transaction

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_sqlalchemy = True
        conf.use_transaction_manager = True
        conf['sqlalchemy.url'] = 'sqlite://'
        conf.commit_veto = custom_commit_veto

        app = conf.make_wsgi_app()
        app = TestApp(app)

        app.get('/test')
        assert fake_transaction.aborted == False

        try:
            app.get('/crash')
        except:
            pass
        assert fake_transaction.aborted == True

        app.get('/forbidden', status=403)
        assert fake_transaction.aborted == False

        app.get('/notfound', status=404)
        assert fake_transaction.aborted == True

        transaction.manager = prev_transaction_manager

    def test_old_sqlalchemy_doom(self):
        fake_transaction = FakeTransaction()
        import transaction
        prev_transaction_manager = transaction.manager
        transaction.manager = fake_transaction

        class RootController(TGController):
            @expose()
            def test(self):
                fake_transaction.doom()
                return 'HI!'

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_sqlalchemy = True
        conf.use_transaction_manager = True
        conf['sqlalchemy.url'] = 'sqlite://'

        app = conf.make_wsgi_app()
        app = TestApp(app)

        app.get('/test')
        assert fake_transaction.aborted == True

        transaction.manager = prev_transaction_manager

    def test_old_sqlalchemy_retry(self):
        fake_transaction = FakeTransaction()
        import transaction
        prev_transaction_manager = transaction.manager
        transaction.manager = fake_transaction

        from transaction.interfaces import TransientError

        class RootController(TGController):
            attempts = []

            @expose()
            def test(self):
                self.attempts.append(True)
                if len(self.attempts) == 3:
                    return 'HI!'
                raise TransientError()

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_sqlalchemy = True
        conf.use_transaction_manager = True
        conf['sqlalchemy.url'] = 'sqlite://'
        conf['tm.attempts'] = 3

        app = conf.make_wsgi_app()
        app = TestApp(app)

        resp = app.get('/test')
        assert 'HI' in resp

        transaction.manager = prev_transaction_manager

    def test_old_sqlalchemy_is_disabled_when_missing(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_sqlalchemy = False
        conf.use_transaction_manager = True

        app = conf.make_wsgi_app()
        assert conf.use_transaction_manager is False

    def test_setup_persistence_custom_backward_compatible(self):
        self.config.package = PackageWithModel()

        def custom_persistence(conf):
            conf['gotcha'] = 'YES!'

        object.__setattr__(self.config, '_setup_persistence', custom_persistence)
        conf = self.config.make_load_environment()({}, {})
        assert conf['gotcha'] == 'YES!', conf

    def test_setup_sqla_persistance(self):
        self.config['sqlalchemy.url'] = 'sqlite://'
        self.config.use_sqlalchemy = True

        self.config.package = PackageWithModel()
        self.config._setup_persistence(self.config._init_config({}, {}))

    def test_setup_sqla_balanced(self):
        self.config['sqlalchemy.master.url'] = 'sqlite://'
        self.config['sqlalchemy.slaves.slave1.url'] = 'sqlite://'
        self.config.use_sqlalchemy = True

        self.config.package = PackageWithModel()
        self.config._setup_persistence(self.config._init_config({}, {}))

    @raises(TGConfigError)
    def test_setup_sqla_balanced_prevent_slave_named_master(self):
        self.config['sqlalchemy.master.url'] = 'sqlite://'
        self.config['sqlalchemy.slaves.master.url'] = 'sqlite://'
        self.config.use_sqlalchemy = True

        self.config.package = PackageWithModel()
        self.config._setup_persistence(self.config._init_config({}, {}))

    @raises(TGConfigError)
    def test_setup_sqla_balanced_no_slaves(self):
        self.config['sqlalchemy.master.url'] = 'sqlite://'
        self.config.use_sqlalchemy = True

        self.config.package = PackageWithModel()
        self.config._setup_persistence(self.config._init_config({}, {}))

    def test_setup_ming_persistance(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_ming = True
        conf['ming.url'] = 'mim://'
        conf['ming.db'] = 'inmemdb'

        app = conf.make_wsgi_app()

        tgapp = app.application
        while not isinstance(tgapp, TGApp):
            tgapp = tgapp.app

        ming_handler = tgapp.wrapped_dispatch
        while ming_handler != tgapp._dispatch:
            if isinstance(ming_handler, MingApplicationWrapper):
                break
            ming_handler = ming_handler.next_handler
        assert isinstance(ming_handler, MingApplicationWrapper), ming_handler

        class FakeMingSession(object):
            actions = []

            def flush_all(self):
                self.actions.append('FLUSH')

            def close_all(self):
                self.actions.append('CLOSE')

        ming_handler.ThreadLocalODMSession = FakeMingSession()

        app = TestApp(app)
        resp = app.get('/test')
        assert 'HI' in resp

        assert ming_handler.ThreadLocalODMSession.actions == ['FLUSH']

    def test_setup_ming_persistance_closes_on_failure(self):
        class RootController(TGController):
            @expose()
            def test(self):
                raise Exception('CRASH!')

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_ming = True
        conf['ming.url'] = 'mim://'
        conf['ming.db'] = 'inmemdb'

        app = conf.make_wsgi_app()

        tgapp = app.application
        while not isinstance(tgapp, TGApp):
            tgapp = tgapp.app

        ming_handler = tgapp.wrapped_dispatch
        while ming_handler != tgapp._dispatch:
            if isinstance(ming_handler, MingApplicationWrapper):
                break
            ming_handler = ming_handler.next_handler
        assert isinstance(ming_handler, MingApplicationWrapper), ming_handler

        class FakeMingSession(object):
            actions = []

            def flush_all(self):
                self.actions.append('FLUSH')

            def close_all(self):
                self.actions.append('CLOSE')

        ming_handler.ThreadLocalODMSession = FakeMingSession()

        app = TestApp(app)

        try:
            app.get('/test', status=500)
        except:
            assert ming_handler.ThreadLocalODMSession.actions == ['CLOSE']
        else:
            assert False, 'Should have raised exception'

    def test_setup_ming_persistance_with_url_alone(self):
        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=None)
        conf.package = package
        conf.model = package.model
        conf.use_ming = True
        conf['ming.url'] = 'mim://inmemdb'

        app = conf.make_wsgi_app()
        assert app is not None

        dstore = config['tg.app_globals'].ming_datastore
        dstore_name = dstore.name
        # Looks like ming has empty dstore.name when using MIM.
        assert dstore_name == '', dstore

    def test_setup_sqla_and_ming_both(self):
        package = PackageWithModel()
        base_config = AppConfig(minimal=True, root_controller=None)
        base_config.package = package
        base_config.model = package.model
        base_config.use_ming = True
        base_config['ming.url'] = 'mim://inmemdb'
        base_config.use_sqlalchemy = True
        base_config['sqlalchemy.url'] = 'sqlite://'

        app = base_config.make_wsgi_app()
        assert app is not None

        assert config['MingSession'], config
        assert config['tg.app_globals'].ming_datastore, config['tg.app_globals']

        assert config['SQLASession'], config
        assert config['tg.app_globals'].sa_engine, config['tg.app_globals']

        assert config['DBSession'] is config['SQLASession'], config

    def test_setup_ming_persistance_with_url_and_db(self):
        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=None)
        conf.package = package
        conf.model = package.model
        conf.use_ming = True
        conf['ming.url'] = 'mim://inmemdb'
        conf['ming.db'] = 'realinmemdb'

        app = conf.make_wsgi_app()
        assert app is not None

        dstore = config['tg.app_globals'].ming_datastore
        dstore_name = dstore.name
        assert dstore_name == 'realinmemdb', dstore

    def test_setup_ming_persistance_advanced_options(self):
        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=None)
        conf.package = package
        conf.model = package.model
        conf.use_ming = True
        conf['ming.url'] = 'mim://inmemdb'
        conf['ming.connection.read_preference'] = 'PRIMARY'

        app = conf.make_wsgi_app()
        assert app is not None

    def test_setup_ming_persistance_replica_set(self):
        if sys.version_info[:2] == (2, 6):
            raise SkipTest()

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=None)
        conf.package = package
        conf.model = package.model
        conf.use_ming = True
        conf['ming.url'] = 'mongodb://localhost:27017,localhost:27018/testdb?replicaSet=test'
        conf['ming.db'] = ''

        app = conf.make_wsgi_app()
        assert app is not None

        expected_url = 'mongodb://localhost:27017,localhost:27018/?replicaSet=test'
        expected_db = 'testdb'

        dstore = config['tg.app_globals'].ming_datastore
        assert expected_db == dstore.name, dstore.name
        assert expected_url == dstore.bind._conn_args[0], dstore.bind._conn_args

    def test_setup_mig_persistance_replica_set_option(self):
        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=None)
        conf.package = package
        conf.model = package.model
        conf.use_ming = True
        conf['ming.url'] = 'mongodb://localhost:27017,localhost:27018/testdb'
        conf['ming.connection.replicaSet'] = 'test'
        conf['ming.db'] = ''

        app = conf.make_wsgi_app()
        assert app is not None

        expected_url = 'mongodb://localhost:27017,localhost:27018/'
        expected_db = 'testdb'

        dstore = config['tg.app_globals'].ming_datastore
        assert expected_db == dstore.name, dstore.name
        assert expected_url == dstore.bind._conn_args[0], dstore.bind._conn_args
        assert 'test' == dstore.bind._conn_kwargs.get('replicaSet'), dstore.bind._conn_kwargs

    def test_add_auth_middleware(self):
        class Dummy:pass

        self.config.sa_auth.dbsession = Dummy()
        self.config.sa_auth.user_class = Dummy
        self.config.sa_auth.group_class = Dummy
        self.config.sa_auth.permission_class = Dummy
        self.config.sa_auth.cookie_secret = 'dummy'
        self.config.sa_auth.password_encryption_method = 'sha'

        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

    def test_add_static_file_middleware(self):
        self.config._add_static_file_middleware(self.config._init_config({}, {}), None)

    def test_setup_sqla_auth(self):
        if PY3: raise SkipTest()

        class RootController(TGController):
            @expose()
            def test(self):
                return str(request.environ)

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.use_sqlalchemy = True
        conf.auth_backend = 'sqlalchemy'
        conf['sa_auth'] = {'authmetadata': ApplicationAuthMetadata(),
                           'dbsession': None,
                           'user_class': None,
                           'cookie_secret': '12345'}
        conf['sqlalchemy.url'] = 'sqlite://'
        app = conf.make_wsgi_app()
        app = TestApp(app)

        resp = app.get('/test')
        assert 'repoze.who.plugins' in resp, resp

        self.config.auth_backend = None

    def test_setup_ming_auth(self):
        self.config.auth_backend = 'ming'

        self.config._setup_auth(self.config._init_config({}, {}))
        assert 'sa_auth' in config

        self.config.auth_backend = None

    def test_deprecated_register_hooks(self):
        def dummy(*args):
            pass

        milestones.config_ready._reset()
        milestones.environment_loaded._reset()

        self.config.register_hook('startup', dummy)
        self.config.register_hook('shutdown', dummy)
        self.config.register_hook('controller_wrapper', dummy)
        for hook_name in ('before_validate', 'before_call', 'before_render',
                          'after_render', 'before_render_call', 'after_render_call',
                          'before_config', 'after_config'):
            self.config.register_hook(hook_name, dummy)

        milestones.config_ready.reach()
        milestones.environment_loaded.reach()

        for hooks in ('before_validate', 'before_call', 'before_render',
                      'after_render', 'before_render_call', 'after_render_call',
                      'before_config', 'after_config', 'startup', 'shutdown'):
            assert tg.hooks._hooks[hooks]

        assert self.config.controller_wrappers

    @raises(TGConfigError)
    def test_missing_secret(self):
        self.config.auth_backend = 'sqlalchemy'
        self.config.pop('session.secret', None)
        self.config._setup_auth(self.config._init_config({}, {}))

    def test_sessions_enabled(self):
        class RootController(TGController):
            @expose('json')
            def test(self):
                try:
                    tg.session['counter'] += 1
                except KeyError:
                    tg.session['counter'] = 0

                tg.session.save()
                return dict(counter=tg.session['counter'])

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['session.enabled'] = True
        app = conf.make_wsgi_app()
        app = TestApp(app)

        resp = app.get('/test')
        assert resp.json['counter'] == 0, resp

        resp = app.get('/test')
        assert resp.json['counter'] == 1, resp

    def test_backware_compatible_sessions_enabled(self):
        class RootController(TGController):
            @expose('json')
            def test(self):
                try:
                    tg.session['counter'] += 1
                except KeyError:
                    tg.session['counter'] = 0

                tg.session.save()
                return dict(counter=tg.session['counter'])

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['session.enabled'] = False
        conf['use_session_middleware'] = True
        app = conf.make_wsgi_app()
        app = TestApp(app)

        resp = app.get('/test')
        assert resp.json['counter'] == 0, resp

        resp = app.get('/test')
        assert resp.json['counter'] == 1, resp

    def test_caching_enabled(self):
        class RootController(TGController):
            @expose('json')
            def test(self):
                cache = tg.cache.get_cache('test_caching_enabled')
                now = cache.get_value('test_cache_key', createfunc=datetime.utcnow)
                return dict(now=now)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['cache.enabled'] = True
        app = conf.make_wsgi_app()
        app = TestApp(app)

        resp = app.get('/test')
        now = resp.json['now']

        for x in range(20):
            resp = app.get('/test')
            assert resp.json['now'] == now, (resp, now)

    def test_backward_compatible_caching_enabled(self):
        class RootController(TGController):
            @expose('json')
            def test(self):
                cache = tg.cache.get_cache('test_caching_enabled')
                now = cache.get_value('test_cache_key', createfunc=datetime.utcnow)
                return dict(now=now)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['cache.enabled'] = False
        conf['use_cache_middleware'] = True
        app = conf.make_wsgi_app()
        app = TestApp(app)

        resp = app.get('/test')
        now = resp.json['now']

        for x in range(20):
            resp = app.get('/test')
            assert resp.json['now'] == now, (resp, now)

    def test_controler_wrapper_setup(self):
        orig_caller = self.config.controller_caller
        self.config.controller_wrappers = []
        self.config._setup_controller_wrappers(self.config._init_config({}, {}))
        assert config['controller_caller'] == orig_caller

        def controller_wrapper(caller):
            def call(*args, **kw):
                return caller(*args, **kw)
            return call

        orig_caller = self.config.controller_caller
        self.config.controller_wrappers = [controller_wrapper]
        self.config._setup_controller_wrappers(self.config._init_config({}, {}))
        assert config['controller_caller'].__name__ == controller_wrapper(orig_caller).__name__

    def test_backward_compatible_controler_wrapper_setup(self):
        orig_caller = self.config.controller_caller
        self.config.controller_wrappers = []
        self.config._setup_controller_wrappers(self.config._init_config({}, {}))
        assert config['controller_caller'] == orig_caller

        def controller_wrapper(app_config, caller):
            def call(*args, **kw):
                return caller(*args, **kw)
            return call

        orig_caller = self.config.controller_caller
        self.config.controller_wrappers = [controller_wrapper]
        self.config._setup_controller_wrappers(self.config._init_config({}, {}))

        deprecated_wrapper = config['controller_caller'].wrapper
        assert deprecated_wrapper.__name__ == controller_wrapper(self.config, orig_caller).__name__

    def test_global_controller_wrapper(self):
        milestones._reset_all()

        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        wrapper_has_been_visited = []
        def controller_wrapper(caller):
            def call(*args, **kw):
                wrapper_has_been_visited.append(True)
                return caller(*args, **kw)
            return call

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.register_hook('controller_wrapper', controller_wrapper)
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        assert 'HI!' in app.get('/test')
        assert wrapper_has_been_visited[0] is True

    def test_backward_compatible_global_controller_wrapper(self):
        milestones._reset_all()

        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        wrapper_has_been_visited = []
        def controller_wrapper(app_config, caller):
            def call(*args, **kw):
                wrapper_has_been_visited.append(True)
                return caller(*args, **kw)
            return call

        def controller_wrapper2(app_config, caller):
            def call(controller, remainder, params):
                wrapper_has_been_visited.append(True)
                return caller(controller, remainder, params)
            return call

        def controller_wrapper3(caller):
            def call(config, controller, remainder, params):
                wrapper_has_been_visited.append(True)
                return caller(config, controller, remainder, params)
            return call

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.register_hook('controller_wrapper', controller_wrapper2)
        conf.register_hook('controller_wrapper', controller_wrapper3)
        conf.register_hook('controller_wrapper', controller_wrapper)
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        assert 'HI!' in app.get('/test')
        assert len(wrapper_has_been_visited) == 3

    def test_dedicated_controller_wrapper(self):
        milestones._reset_all()

        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        wrapper_has_been_visited = []
        def controller_wrapper(caller):
            def call(*args, **kw):
                wrapper_has_been_visited.append(True)
                return caller(*args, **kw)
            return call

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.register_controller_wrapper(controller_wrapper, controller=RootController.test)
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        assert 'HI!' in app.get('/test')
        assert wrapper_has_been_visited[0] is True

    def test_dedicated_controller_wrapper_old(self):
        milestones._reset_all()

        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        wrapper_has_been_visited = []
        def controller_wrapper(caller):
            def call(*args, **kw):
                wrapper_has_been_visited.append(True)
                return caller(*args, **kw)
            return call

        conf = AppConfig(minimal=True, root_controller=RootController())
        tg.hooks.wrap_controller(controller_wrapper, controller=RootController.test)
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        assert 'HI!' in app.get('/test')
        assert wrapper_has_been_visited[0] is True

    def test_mixed_controller_wrapper(self):
        milestones._reset_all()

        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        app_wrapper_has_been_visited = []
        def app_controller_wrapper(caller):
            def call(*args, **kw):
                app_wrapper_has_been_visited.append(True)
                return caller(*args, **kw)
            return call

        wrapper_has_been_visited = []
        def controller_wrapper(caller):
            def call(*args, **kw):
                wrapper_has_been_visited.append(True)
                return caller(*args, **kw)
            return call

        conf = AppConfig(minimal=True, root_controller=RootController())
        tg.hooks.wrap_controller(app_controller_wrapper)
        tg.hooks.wrap_controller(controller_wrapper, controller=RootController.test)
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        assert 'HI!' in app.get('/test')
        assert wrapper_has_been_visited[0] is True
        assert app_wrapper_has_been_visited[0] is True

    def test_controler_wrapper_after_environment_setup(self):
        milestones._reset_all()

        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        wrapper_has_been_visited = []
        def controller_wrapper(caller):
            def call(*args, **kw):
                wrapper_has_been_visited.append(True)
                return caller(*args, **kw)
            return call

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.register_controller_wrapper(controller_wrapper)
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        assert 'HI!' in app.get('/test')
        assert wrapper_has_been_visited[0] is True
        assert len(wrapper_has_been_visited) == 1

        conf.register_controller_wrapper(controller_wrapper)
        app2 = conf.make_wsgi_app()
        app2 = TestApp(app2)

        wrapper_has_been_visited[:] = []
        assert 'HI!' in app2.get('/test')
        assert wrapper_has_been_visited[0] is True
        assert len(wrapper_has_been_visited) == 2

    def test_application_wrapper_setup(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        wrapper_has_been_visited = []
        class AppWrapper(object):
            def __init__(self, dispatcher):
                self.dispatcher = dispatcher
            def __call__(self, *args, **kw):
                wrapper_has_been_visited.append(True)
                return self.dispatcher(*args, **kw)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.register_wrapper(AppWrapper)
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        assert 'HI!' in app.get('/test')
        assert wrapper_has_been_visited[0] == True

    def test_application_wrapper_ordering_after(self):
        class AppWrapper1:
            pass
        class AppWrapper2:
            pass
        class AppWrapper3:
            pass
        class AppWrapper4:
            pass
        class AppWrapper5:
            pass

        conf = AppConfig(minimal=True)
        conf.register_wrapper(AppWrapper2)
        conf.register_wrapper(AppWrapper4, after=AppWrapper3)
        conf.register_wrapper(AppWrapper3)
        conf.register_wrapper(AppWrapper1, after=False)
        conf.register_wrapper(AppWrapper5, after=AppWrapper3)
        milestones.environment_loaded.reach()

        app_wrappers = list(conf.application_wrappers.values())
        assert app_wrappers[0] == AppWrapper1
        assert app_wrappers[1] == AppWrapper2
        assert app_wrappers[2] == AppWrapper3
        assert app_wrappers[3] == AppWrapper4
        assert app_wrappers[4] == AppWrapper5

    def test_wrap_app(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        middleware_has_been_visited = []
        class AppWrapper(object):
            def __init__(self, app):
                self.app = app
            def __call__(self, environ, start_response):
                middleware_has_been_visited.append(True)
                return self.app(environ, start_response)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app(wrap_app=AppWrapper)
        app = TestApp(app)

        assert 'HI!' in app.get('/test')
        assert middleware_has_been_visited[0] == True

    def test_unsupported_renderer(self):
        renderers = self.config.renderers
        self.config.renderers = ['unknwon']
        try:
            self.config._setup_renderers(self.config._init_config({}, {}))
        except TGConfigError:
            self.config.renderers = renderers
        else:
            assert False

    @raises(TGConfigError)
    def test_cookie_secret_required(self):
        self.config.sa_auth = {}
        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

    def test_sqla_auth_middleware(self):
        if PY3: raise SkipTest()

        self.config.auth_backend = 'sqlalchemy'
        self.config.skip_authentication = True
        self.config['sa_auth'] = {'authmetadata': ApplicationAuthMetadata(),
                                  'dbsession': None,
                                  'user_class':None,
                                  'cookie_secret':'12345',
                                  'authenticators':UncopiableList([('default', None)])}
        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

        authenticators = [x[0] for x in cfg['sa_auth']['authenticators']]
        assert 'cookie' in authenticators
        assert 'sqlauth' in authenticators

        self.config['sa_auth'] = {}
        self.config.auth_backend = None

    def test_sqla_auth_middleware_using_translations(self):
        if PY3: raise SkipTest()

        self.config.auth_backend = 'sqlalchemy'
        self.config['sa_auth'] = {'authmetadata': ApplicationAuthMetadata(),
                                  'dbsession': None,
                                  'user_class':None,
                                  'translations': {'user_name':'SomethingElse'},
                                  'cookie_secret':'12345',
                                  'authenticators':UncopiableList([('default', None)])}
        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

        authenticators = [x[0] for x in self.config['sa_auth']['authenticators']]
        assert 'cookie' in authenticators
        assert 'sqlauth' in authenticators

        auth = None
        for authname, authobj in self.config['sa_auth']['authenticators']:
            if authname == 'sqlauth':
                auth = authobj
                break

        assert auth is not None, self.config['sa_auth']['authenticators']
        assert auth.translations['user_name'] == 'SomethingElse', auth.translations

        self.config['sa_auth'] = {}
        self.config.auth_backend = None

    def test_sqla_auth_middleware_default_after(self):
        if PY3: raise SkipTest()

        self.config.auth_backend = 'sqlalchemy'
        self.config['sa_auth'] = {'authmetadata': ApplicationAuthMetadata(),
                                  'cookie_secret':'12345',
                                  'dbsession': None,
                                  'user_class': None,
                                  'authenticators':UncopiableList([('superfirst', None),
                                                                   ('default', None)])}

        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

        authenticators = [x[0] for x in self.config['sa_auth']['authenticators']]
        assert authenticators[1] == 'superfirst'
        assert 'cookie' in authenticators
        assert 'sqlauth' in authenticators

        self.config['sa_auth'] = {}
        self.config.auth_backend = None

    def test_sqla_auth_middleware_no_authenticators(self):
        if PY3: raise SkipTest()

        self.config.auth_backend = 'sqlalchemy'
        self.config['sa_auth'] = {'authmetadata': ApplicationAuthMetadata(),
                                  'dbsession': None,
                                  'user_class': None,
                                  'cookie_secret':'12345'}

        #In this case we can just test it doesn't crash
        #as the sa_auth dict doesn't have an authenticators key to check for
        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

        self.config['sa_auth'] = {}
        self.config.auth_backend = None

    def test_sqla_auth_middleware_only_mine(self):
        past_config_sa_auth = config.sa_auth
        config.sa_auth = {}

        class RootController(TGController):
            @expose()
            def test(self):
                return str(request.environ)

            @expose()
            def forbidden(self):
                response.status = "401"

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = package
        conf.model = package.model
        conf.auth_backend = 'sqlalchemy'
        conf.use_sqlalchemy = True
        conf['sqlalchemy.url'] = 'sqlite://'

        alwaysadmin = _AuthenticationForgerPlugin(fake_user_key='FAKE_USER')
        conf['sa_auth'] = {'authmetadata': ApplicationAuthMetadata(),
                           'cookie_secret':'12345',
                           'form_plugin':alwaysadmin,
                           'authenticators':UncopiableList([('alwaysadmin', alwaysadmin)]),
                           'identifiers':[('alwaysadmin', alwaysadmin)],
                           'challengers':[]}

        app = conf.make_wsgi_app()

        authenticators = [x[0] for x in conf['sa_auth']['authenticators']]
        assert authenticators[0] == 'alwaysadmin'
        assert 'sqlauth' not in authenticators

        challengers = [x[1] for x in conf['sa_auth']['challengers']]
        assert alwaysadmin in challengers

        app = TestApp(app)
        assert 'repoze.who.identity' in app.get('/test', extra_environ={'FAKE_USER':'admin'})
        assert app.get('/forbidden', status=401)

        self.config['sa_auth'] = {}
        self.config.auth_backend = None
        config.sa_auth = past_config_sa_auth

    def test_sqla_auth_logging_stderr(self):
        past_config_sa_auth = config.sa_auth
        config.sa_auth = {}

        package = PackageWithModel()
        conf = AppConfig(minimal=True, root_controller=None)
        conf.package = package
        conf.model = package.model
        conf.auth_backend = 'sqlalchemy'
        conf.use_sqlalchemy = True
        conf['sqlalchemy.url'] = 'sqlite://'

        alwaysadmin = _AuthenticationForgerPlugin(fake_user_key='FAKE_USER')
        conf['sa_auth'] = {'authmetadata': ApplicationAuthMetadata(),
                           'cookie_secret':'12345',
                           'form_plugin':alwaysadmin,
                           'log_level':'DEBUG',
                           'authenticators':UncopiableList([('alwaysadmin', alwaysadmin)]),
                           'identifiers':[('alwaysadmin', alwaysadmin)],
                           'challengers':[]}

        conf['sa_auth']['log_file'] = 'stderr'
        app = conf.make_wsgi_app()
        conf['sa_auth']['log_file'] = 'stdout'
        app = conf.make_wsgi_app()

        import tempfile
        f = tempfile.NamedTemporaryFile()
        conf['sa_auth']['log_file'] = f.name
        app = conf.make_wsgi_app()

        self.config['sa_auth'] = {}
        self.config.auth_backend = None
        config.sa_auth = past_config_sa_auth

    def test_ming_auth_middleware(self):
        if PY3: raise SkipTest()

        self.config.auth_backend = 'ming'
        self.config['sa_auth'] = {'authmetadata': ApplicationAuthMetadata(),
                                  'user_class':None,
                                  'cookie_secret':'12345',
                                  'authenticators':UncopiableList([('default', None)])}
        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

        authenticators = [x[0] for x in self.config['sa_auth']['authenticators']]
        assert 'cookie' in authenticators
        assert 'mingauth' in authenticators

        self.config['sa_auth'] = {}
        self.config.auth_backend = None

    @raises(KeyError)
    def test_sqla_auth_middleware_no_backend(self):
        #This is expected to raise error as no authenticators are specified for a custom backend
        past_config_sa_auth = config.sa_auth
        config.sa_auth = {}

        self.config.auth_backend = None
        self.config['sa_auth'] = {'authmetadata': ApplicationAuthMetadata(),
                                  'cookie_secret':'12345'}
        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

        authenticators = [x[0] for x in self.config['sa_auth']['authenticators']]
        assert 'cookie' in authenticators
        assert len(authenticators) == 1

        self.config['sa_auth'] = {}
        self.config.auth_backend = None
        config.sa_auth = past_config_sa_auth

    def test_tgauthmetadata_auth_middleware(self):
        self.config.auth_backend = 'sqlalchemy'
        self.config['sa_auth'] = {'authmetadata': ApplicationAuthMetadataWithAuthentication(),
                                  'dbsession': None,
                                  'user_class':None,
                                  'cookie_secret':'12345',
                                  'authenticators':UncopiableList([('default', None)])}
        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

        authenticators = [x[0] for x in self.config['sa_auth']['authenticators']]
        assert 'cookie' in authenticators
        assert 'tgappauth' in authenticators

        self.config['sa_auth'] = {}
        self.config.auth_backend = None

    def test_auth_setup_default_identifier(self):
        self.config.auth_backend = 'sqlalchemy'
        self.config['sa_auth'] = {'authmetadata': ApplicationAuthMetadataWithAuthentication(),
                                  'dbsession': None,
                                  'user_class':None,
                                  'cookie_secret':'12345',
                                  'identifiers': UncopiableList([('default', None)])}
        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

        identifiers = [x[0] for x in self.config['sa_auth']['identifiers']]
        assert 'cookie' in identifiers

        self.config['sa_auth'] = {}
        self.config.auth_backend = None

    def test_auth_setup_custom_identifier(self):
        self.config.auth_backend = 'sqlalchemy'
        self.config['sa_auth'] = {'authmetadata': ApplicationAuthMetadataWithAuthentication(),
                                  'dbsession': None,
                                  'user_class':None,
                                  'cookie_secret':'12345',
                                  'identifiers': UncopiableList([('custom', None)])}
        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

        identifiers = [x[0] for x in self.config['sa_auth']['identifiers']]
        assert 'custom' in identifiers

        self.config['sa_auth'] = {}
        self.config.auth_backend = None

    def test_auth_middleware_doesnt_touch_authenticators(self):
        # Checks that the auth middleware process doesn't touch original authenticators
        # list, to prevent regressions on this.
        self.config.auth_backend = 'sqlalchemy'
        self.config['sa_auth'] = {'authmetadata': ApplicationAuthMetadataWithAuthentication(),
                                  'dbsession': None,
                                  'user_class':None,
                                  'cookie_secret':'12345',
                                  'authenticators':[('default', None)]}
        cfg = self.config._init_config({}, {})
        self.config._setup_auth(cfg)
        self.config._add_auth_middleware(cfg, None)

        authenticators = [x[0] for x in self.config['sa_auth']['authenticators']]
        assert len(authenticators) == 1

        self.config['sa_auth'] = {}
        self.config.auth_backend = None

    def test_tgauthmetadata_loginpwd(self):
        who_authenticator = _AuthMetadataAuthenticator(ApplicationAuthMetadataWithAuthentication(), using_password=True)
        assert who_authenticator.authenticate({}, {}) == None

    def test_tgauthmetadata_nologinpwd(self):
        who_authenticator = _AuthMetadataAuthenticator(ApplicationAuthMetadataWithAuthentication(), using_password=False)
        assert who_authenticator.authenticate({}, {}) == 1

    def test_toscawidgets_recource_variant(self):
        if PY3: raise SkipTest()

        resultingconfig = {}

        def fake_make_middleware(app, twconfig):
            resultingconfig.update(twconfig)
            return app

        import tw.api
        prev_tw_make_middleware = tw.api.make_middleware

        tw.api.make_middleware = fake_make_middleware
        config['toscawidgets.framework.resource_variant'] = 'min'
        self.config._add_tosca_middleware(config, None)
        config.pop('toscawidgets.framework.resource_variant', None)

    def test_error_middleware_disabled_with_optimize(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = PackageWithModel()

        os.environ['PYTHONOPTIMIZE'] = '2'
        app = conf.make_wsgi_app()
        os.environ.pop('PYTHONOPTIMIZE')

        app = TestApp(app)
        assert 'HI!' in app.get('/test')

    def test_serve_statics(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = PackageWithModel()
        conf.serve_static = True
        app = conf.make_wsgi_app()
        assert app.__class__.__name__.startswith('Statics')

        app = TestApp(app)
        assert 'HI!' in app.get('/test')

    def test_mount_point_with_minimal(self):
        class SubController(TGController):
            @expose()
            def test(self):
                return self.mount_point

        class RootController(TGController):
            sub = SubController()

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()

        app = TestApp(app)
        assert '/sub' in app.get('/sub/test')

    def test_application_test_vars(self):
        class RootController(TGController):
            pass

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        assert 'DONE' in app.get('/_test_vars')
        assert request.path == '/_test_vars'

        # This should trash away the preserved registry to avoid
        # leaking memory.
        app.get('/', status=404)

        try:
            request.path
        except TypeError:
            # TypeError means the request has been properly removed
            pass
        else:
            assert False, 'There should have been no requests in place...'

    def test_application_empty_controller(self):
        class RootController(object):
            def __call__(self, environ, start_response):
                return None

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        try:
            r = app.get('/something')
        except Exception as e:
            assert 'No content returned by controller' in str(e)
        else:
            assert False, 'Should have raised "No content returned by controller"'

    def test_application_test_mode_detection(self):
        class FakeRegistry(object):
            def register(self, *args, **kw):
                pass

        a = TGApp()
        testmode, __, __ = a._setup_app_env({'paste.registry':FakeRegistry()})
        assert testmode is False

        testmode, __, __ = a._setup_app_env({'paste.registry':FakeRegistry(),
                                             'paste.testing_variables':{}})
        assert testmode is True

    def test_application_no_controller_hijacking(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        class AppWrapper(object):
            def __init__(self, dispatcher):
                self.dispatcher = dispatcher
            def __call__(self, controller, environ, start_response):
                return self.dispatcher(None, environ, start_response)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.register_wrapper(AppWrapper)
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        app.get('/test', status=404)

    def test_package_no_app_globals(self):
        class RootController(TGController):
            pass

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.package = sys.modules[__name__]

        app = conf.make_wsgi_app()

    def test_custom_error_document(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                abort(403)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['errorpage.enabled'] = True
        conf['errorpage.handle_exceptions'] = False
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=403)
        assert 'ERROR!!!' in resp, resp

    def test_custom_error_document_with_streamed_response(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                response.status_code = 403
                def _output():
                    yield 'Hi'
                    yield 'World'
                return _output()

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['errorpage.enabled'] = True
        conf['errorpage.handle_exceptions'] = False
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=403)
        assert 'ERROR!!!' in resp, resp

    def test_error_document_passthrough(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                request.disable_error_pages()
                abort(403, detail='Custom Detail')

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['errorpage.enabled'] = True
        conf['errorpage.handle_exceptions'] = False
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=403)
        assert 'Custom Detail' in resp, resp

    def test_custom_old_error_document(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                abort(403)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['errorpage.enabled'] = True
        conf.status_code_redirect = True
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=403)
        assert 'ERROR!!!' in resp, resp

    def test_custom_old_error_document_with_streamed_response(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                response.status_code = 403
                def _output():
                    yield 'Hi'
                    yield 'World'
                return _output()

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['errorpage.enabled'] = True
        conf.status_code_redirect = True
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=403)
        assert 'ERROR!!!' in resp, resp

    def test_custom_500_document(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                abort(500)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['errorpage.enabled'] = True
        conf['debug'] = False
        conf['errorpage.handle_exceptions'] = False
        conf['errorpage.status_codes'] += [500]
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=500)
        assert 'ERROR!!!' in resp, resp

    def test_custom_500_document_on_crash(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                raise Exception('Crash!')

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['errorpage.enabled'] = True
        conf['debug'] = False
        conf['errorpage.handle_exceptions'] = True
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=500)
        assert 'ERROR!!!' in resp, resp

    def test_errorpage_reraises_exceptions(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                raise Exception('Crash!')

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['errorpage.enabled'] = True
        conf['debug'] = False
        conf['errorpage.handle_exceptions'] = False
        app = conf.make_wsgi_app(full_stack=False)
        app = TestApp(app)

        try:
            resp = app.get('/test', status=500)
        except Exception as e:
            assert 'Crash!' in str(e)
        else:
            assert False, 'Should have raised Crash! exception'

    def test_old_custom_500_document(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                abort(500)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['debug'] = False
        conf.status_code_redirect = True
        conf['errorpage.enabled'] = True
        conf['errorpage.status_codes'] += [500]
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=500)
        assert 'ERROR!!!' in resp, resp

    def test_skips_custom_500_document_when_debug(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                abort(500)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['errorpage.enabled'] = True
        conf['debug'] = True
        conf['errorpage.handle_exceptions'] = False
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=500)
        assert 'ERROR!!!' not in resp, resp

    def test_skips_old_custom_500_document_when_debug(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                abort(500)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['debug'] = True
        conf.status_code_redirect = True
        conf['errorpage.enabled'] = True
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=500)
        assert 'ERROR!!!' not in resp, resp

    def test_skips_custom_error_document_when_disabled(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                abort(403)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['errorpage.enabled'] = False
        conf['errorpage.status_codes'] = (403, 404)
        conf['errorpage.handle_exceptions'] = False
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=403)
        assert 'ERROR!!!' not in resp, resp

    def test_skips_custom_error_document_when_disabled_and_manually_registered(self):
        class ErrorController(TGController):
            @expose()
            def document(self, *args, **kw):
                return 'ERROR!!!'

        class RootController(TGController):
            error = ErrorController()
            @expose()
            def test(self):
                abort(403)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.register_wrapper(ErrorPageApplicationWrapper)
        conf['errorpage.enabled'] = False
        conf['errorpage.status_codes'] = (403, 404)
        conf['errorpage.handle_exceptions'] = False
        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=403)
        assert 'ERROR!!!' not in resp, resp

    def test_errorware_configuration(self):
        class RootController(TGController):
            @expose()
            def test(self, *args, **kwargs):
                return 'HI'

        conf = AppConfig(minimal=True, root_controller=RootController())
        app = conf.make_wsgi_app(global_conf={'trace_errors.error_email': 'test@domain.com'},
                                 full_stack=True)
        app = TestApp(app)

        resp = app.get('/test')
        assert 'HI' in resp, resp

        assert config['tg.errorware']['error_email'] == 'test@domain.com'
        assert config['tg.errorware']['error_subject_prefix'] == 'WebApp Error: '
        assert config['tg.errorware']['error_message'] == 'An internal server error occurred'

    def test_tw2_unsupported_renderer(self):
        import tw2.core

        class RootController(TGController):
            @expose()
            def test(self, *args, **kwargs):
                rl = tw2.core.core.request_local()
                tw2conf = rl['middleware'].config
                return ','.join(tw2conf.preferred_rendering_engines)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.prefer_toscawidgets2 = True
        conf.renderers = ['json', 'genshi']
        conf.default_renderer = 'json'

        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test')
        assert 'genshi' in resp, resp

    def test_tw2_renderers_preference(self):
        import tw2.core

        class RootController(TGController):
            @expose()
            def test(self, *args, **kwargs):
                rl = tw2.core.core.request_local()
                tw2conf = rl['middleware'].config
                return ','.join(tw2conf.preferred_rendering_engines)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.prefer_toscawidgets2 = True
        conf.renderers = ['genshi']
        conf.default_renderer = 'genshi'

        app = conf.make_wsgi_app(full_stack=True)
        app = TestApp(app)

        resp = app.get('/test')
        assert 'genshi' in resp, resp

    def test_tw2_unsupported(self):
        import tw2.core

        class RootController(TGController):
            @expose()
            def test(self, *args, **kwargs):
                rl = tw2.core.core.request_local()
                tw2conf = rl['middleware'].config
                return ','.join(tw2conf.preferred_rendering_engines)

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.prefer_toscawidgets2 = True
        conf.renderers = ['json']
        conf.default_renderer = 'json'

        try:
            app = conf.make_wsgi_app(full_stack=True)
            assert False
        except TGConfigError as e:
            assert 'None of the configured rendering engines is supported' in str(e)

    def test_backward_compatible_engine_failed_setup(self):
        class RootController(TGController):
            @expose()
            def test(self, *args, **kwargs):
                return 'HELLO'

        def setup_broken_renderer():
            return False

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.setup_broken_renderer = setup_broken_renderer
        conf.renderers = ['json', 'broken']

        app = conf.make_wsgi_app(full_stack=True)
        assert conf.renderers == ['json']

    def test_backward_compatible_engine_success_setup(self):
        class RootController(TGController):
            @expose()
            def test(self, *args, **kwargs):
                return 'HELLO'

        conf = AppConfig(minimal=True, root_controller=RootController())

        def setup_broken_renderer():
            conf.render_functions.broken = 'BROKEN'
            return True

        conf.setup_broken_renderer = setup_broken_renderer
        conf.renderers = ['json', 'broken']

        app = conf.make_wsgi_app(full_stack=True)
        assert conf.renderers == ['json', 'broken']
        assert conf.render_functions.broken == 'BROKEN'

    def test_render_factory_success(self):
        class RootController(TGController):
            @expose()
            def test(self, *args, **kwargs):
                return 'HELLO'

        class FailedFactory(RendererFactory):
            engines = {'broken': {'content_type': 'text/plain'}}

            @classmethod
            def create(cls, config, app_globals):
                return {'broken': 'BROKEN'}

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.register_rendering_engine(FailedFactory)
        conf.renderers = ['json', 'broken']

        app = conf.make_wsgi_app(full_stack=True)
        assert conf.renderers == ['json', 'broken']
        assert conf.render_functions.broken == 'BROKEN'

    def test_render_factory_failure(self):
        class RootController(TGController):
            @expose()
            def test(self, *args, **kwargs):
                return 'HELLO'

        class FailedFactory(RendererFactory):
            engines = {'broken': {'content_type': 'text/plain'}}

            @classmethod
            def create(cls, config, app_globals):
                return None

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.register_rendering_engine(FailedFactory)
        conf.renderers = ['json', 'broken']

        app = conf.make_wsgi_app(full_stack=True)
        assert conf.renderers == ['json']

    def test_make_body_seekable(self):
        class RootController(TGController):
            @expose()
            def test(self, *args, **kwargs):
                request.body_file.seek(0)
                return 'HELLO'

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['make_body_seekable'] = True

        app = conf.make_wsgi_app(full_stack=False)
        assert app.application.__class__.__name__ == 'SeekableRequestBodyMiddleware', \
            app.application.__class__

        app = TestApp(app)
        assert 'HELLO' in app.get('/test')

    def test_make_body_seekable_disabled(self):
        class RootController(TGController):
            @expose()
            def test(self, *args, **kwargs):
                request.body_file.seek(0)
                return 'HELLO'

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['make_body_seekable'] = False

        app = conf.make_wsgi_app(full_stack=False)
        app = TestApp(app)
        assert 'HELLO' in app.get('/test')

    def test_debug_middleware(self):
        class RootController(TGController):
            @expose()
            def test(self):
                raise Exception('Crash!')

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf['errorpage.enabled'] = True
        app = conf.make_wsgi_app(global_conf={'debug': True}, full_stack=True)
        app = TestApp(app)

        resp = app.get('/test', status=500)
        assert 'Exception: Crash! // Backlash' in resp, resp

    def test_make_app_with_custom_appglobals(self):
        class RootController(TGController):
            @expose('')
            def test(self, *args, **kwargs):
                return tg.app_globals.TEXT

        class FakeGlobals(Bunch):
            def __init__(self):
                super(FakeGlobals, self).__init__()
                self['TEXT'] = 'HI!'

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.app_globals = FakeGlobals
        app = conf.make_wsgi_app()
        app = TestApp(app)
        assert 'HI!' in app.get('/test')

    def test_make_app_with_appglobals_submodule(self):
        class RootController(TGController):
            @expose('')
            def test(self, *args, **kwargs):
                return tg.app_globals.text

        conf = AppConfig(minimal=True, root_controller=RootController())

        from .fixtures import package_with_helpers_submodule
        conf['package'] = package_with_helpers_submodule
        app = conf.make_wsgi_app()
        app = TestApp(app)
        assert 'HI!!' in app.get('/test')

    def test_make_app_with_custom_helpers(self):
        class RootController(TGController):
            @expose('')
            def test(self, *args, **kwargs):
                return config['helpers'].get_text()

        class FakeHelpers(Bunch):
            @classmethod
            def get_text(cls):
                return 'HI!'

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.helpers = FakeHelpers()
        app = conf.make_wsgi_app()
        app = TestApp(app)
        assert 'HI!' in app.get('/test')

    def test_make_app_with_helpers_submodule(self):
        class RootController(TGController):
            @expose('')
            def test(self, *args, **kwargs):
                return config['helpers'].get_text()

        conf = AppConfig(minimal=True, root_controller=RootController())

        from .fixtures import package_with_helpers_submodule
        conf['package'] = package_with_helpers_submodule
        app = conf.make_wsgi_app()
        app = TestApp(app)
        assert 'HI!!' in app.get('/test')

    def test_make_app_without_load_environment(self):
        class RootController(TGController):
            @expose()
            def test(self, *args, **kwargs):
                return 'Helpers: %s' % tg.config.get('helpers')

        conf = AppConfig(minimal=True, root_controller=RootController())
        cfg = conf._init_config({}, {})  # Manually call init_config otherwise no tg.config is pushed.
        app = conf.setup_tg_wsgi_app(load_environment=None)()
        app = TestApp(app)

        resp = app.get('/test')
        assert resp.text == 'Helpers: None', resp